/*
Copyright Ideaconsult Ltd. (C) 2005-2007 

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*//**
 * <b>Filename</b> FunctionalGroups.java 
 * @author Nina Jeliazkova nina@acad.bg
 * <b>Created</b> 2005-8-8
 * <b>Project</b> toxTree
 * 
 */
package toxTree.query;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.TreeMap;
import java.util.Vector;

import org.openscience.cdk.Atom;
import org.openscience.cdk.AtomContainer;
import org.openscience.cdk.Bond;
import org.openscience.cdk.CDKConstants;
import org.openscience.cdk.Molecule;
import org.openscience.cdk.Ring;
import org.openscience.cdk.RingSet;
import org.openscience.cdk.SetOfAtomContainers;
import org.openscience.cdk.exception.CDKException;
import org.openscience.cdk.exception.InvalidSmilesException;
import org.openscience.cdk.exception.NoSuchAtomException;
import org.openscience.cdk.graph.ConnectivityChecker;
import org.openscience.cdk.graph.PathTools;
import org.openscience.cdk.graph.matrix.ConnectionMatrix;
import org.openscience.cdk.interfaces.IAtom;
import org.openscience.cdk.interfaces.IAtomContainer;
import org.openscience.cdk.interfaces.IBond;
import org.openscience.cdk.interfaces.IMolecule;
import org.openscience.cdk.interfaces.IRing;
import org.openscience.cdk.interfaces.IRingSet;
import org.openscience.cdk.interfaces.ISetOfAtomContainers;
import org.openscience.cdk.interfaces.ISetOfMolecules;
import org.openscience.cdk.isomorphism.UniversalIsomorphismTester;
import org.openscience.cdk.isomorphism.matchers.IQueryAtom;
import org.openscience.cdk.isomorphism.matchers.InverseSymbolSetQueryAtom;
import org.openscience.cdk.isomorphism.matchers.OrderQueryBond;
import org.openscience.cdk.isomorphism.matchers.QueryAtomContainer;
import org.openscience.cdk.isomorphism.matchers.QueryAtomContainerCreator;
import org.openscience.cdk.isomorphism.matchers.SymbolAndChargeQueryAtom;
import org.openscience.cdk.isomorphism.matchers.SymbolQueryAtom;
import org.openscience.cdk.isomorphism.matchers.SymbolSetQueryAtom;
import org.openscience.cdk.isomorphism.matchers.smarts.AnyAtom;
import org.openscience.cdk.isomorphism.matchers.smarts.AnyOrderQueryBond;
import org.openscience.cdk.isomorphism.mcss.RMap;
import org.openscience.cdk.ringsearch.SSSRFinder;
import org.openscience.cdk.smiles.SmilesParser;
import org.openscience.cdk.tools.HydrogenAdder;
import org.openscience.cdk.tools.MFAnalyser;

import toxTree.logging.TTLogger;
import toxTree.tree.AbstractRule;

/**
 * This class provides static methods for various functional groups
 * In addition there are several static methods to manipulate searching  
 * @author Nina Jeliazkova <br>
 * <b>Modified</b> 2005-8-8
 */
public class FunctionalGroups {
	public static TTLogger logger = new TTLogger(FunctionalGroups.class);
	/**
	 * Messages
	 */
	public static final String MSG_HASGROUP = "\thas group \t";
	public static final String MSG_MOLECULEIS = "Molecule is\t";	
	/**
	 * These constants are used in Atom.setProperty(CH3,true) fashion
	 * in order to mark which atoms/bonds belong to the corresponding group
	 * Used by {@link #markMaps(IAtomContainer, IAtomContainer, List)} method 
	 */
	public static final String CH3 = "CH3";    
    public static final String CH2 = "CH2";
    public static final String CH = "CH";
    public static final String C = "C";
    public static final String HYDROCARBON = "hydrocarbon";
    public static final String PRIMARY_AMINE = "Primary amine";
    public static final String SECONDARY_AMINE = "Secondary amine";
    public static final String TERTIARY_AMINE = "tertiary amine";
    public static final String SECONDARY_AMINE_ALIPHATIC = "Aliphatic Secondary amine";
    public static final String CYANO = "cyano";
    public static final String NITRO = "nitro";
    public static final String NNITROSO = "N-nitroso";
    public static final String DIAZO = "diazo";
    public static final String TRIAZENO = "triazeno";
    public static final String CARBOXYLIC_ACID_SALT = "carboxylic acid salt";
    public static final String CARBOXYLIC_ACID = "carboxylic acid";
    public static final String ACETAL = "acetal";
    public static final String ETHER = "ether";
    public static final String METHYLETHER = "methylether";
    public static final String SULPHIDE = "sulphide";
    public static final String MERCAPTAN = "mercaptan";
    public static final String ESTER = "ester";
    public static final String THIOESTER = "thioester";    
    public static final String KETONE = "ketone";
    public static final String KETONE_SIDECHAIN = "ketone_sidechain";
    public static final String ALDEHYDE = "aldehyde";
    public static final String CARBONYL = "carbonyl";
    public static final String ALCOHOL = "alcohol";
    public static final String SULPHONATE = "sulphonate";
    public static final String SULPHAMATE = "sulphamate";
    public static final String SULPHATE = "sulphate";
    public static final String POLYOXYETHYLENE = "polyoxyethylene";
    public static final String HYDROCHLORIDE_OF_AMINE = "hydrochloride of amine";
    public static final String SULPHATE_OF_AMINE = "sulphate of amine";
    public static final String ACETYLENIC = "acetylenic";
    public static final String LACTONE = "lactone";
    public static final String CYCLIC_DIESTER = "cyclic diester";
    public static final String ISOPRENE = "isoprene unit";
    public static final String HYDROXY = "hydroxy";
    public static final String METHOXY = "methoxy";
    public static final String HYDROXY1 = "1'hydroxy";
    public static final String HYDROXYESTERSUBSTITED = "hydroxy ester substituted";
    public static final String QUATERNARY_NITROGEN= "quaternary nitrogen";
    public static final String QUATERNARY_NITROGEN_EXCEPTION= "quaternary nitrogen exception";
    public static final String ANHYDRIDE="ANHYDRIDE";
    public static final String CARBONATE="CARBONATE";
    
    public static final String  RING_NUMBERING="RingNumbering";
    /**
	 * Use DONTMARK constant when building a query
	 * QueryAtom atom;
	 * atom.setProperty(DONTMARK,query.getID());
	 * This is used as a flag that the atom belongs not to the group but to the neighboring radicals
	 * It is also essential for the {@link #detachGroup(IAtomContainer, QueryAtomContainer)} procedure to work, 
	 * as the bond to break is the one with one atom having the property set, while the other has the property unset  
	 */
	public static final String DONTMARK = "DONTMARK";
	//TODO replace current marking with ALLOCATED
	/**
	 * The intention is to use this property to mark if an atom/bond was already associated with a functional group.
	 * It will help to avoid wrong overlapping allocation of different group to the same atom
	 * Finally, marking by setProperty(query.getID(),new Boolean(true)) may be replaced with setProperty(ALLOCATED,query.getID())
	 * perhaps in  v0.04 release :)
	 */
	public static final String ALLOCATED = "ALLOCATED";
        
    
    protected static HydrogenAdder h = new HydrogenAdder();
    /**
     * 
     */
    protected FunctionalGroups() {
        super();
    }
    /**
     * Query for Methyl group.
     * @return
     */
    public static QueryAtomContainer methyl() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(CH3);

        SymbolQueryAtom c = new SymbolQueryAtom(new Atom("C"));
        SymbolQueryAtom h1 = new SymbolQueryAtom(new Atom("H"));
        SymbolQueryAtom h2 = new SymbolQueryAtom(new Atom("H"));
        SymbolQueryAtom h3 = new SymbolQueryAtom(new Atom("H"));        
        query.addAtom(c);
        query.addAtom(h1);
        query.addAtom(h2);        
        query.addAtom(h3);        
        query.addBond(new OrderQueryBond(c, h1, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(c, h2, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(c, h3, CDKConstants.BONDORDER_SINGLE));
        return query;
    }
    /**
     * Query for methoxy group.
     * @return
     */
    public static QueryAtomContainer methoxy() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(METHOXY);
        AnyAtom r = new AnyAtom(); r.setSymbol("R");
        SymbolQueryAtom o = new SymbolQueryAtom(new Atom("O"));
        SymbolQueryAtom c = new SymbolQueryAtom(new Atom("C"));
        SymbolQueryAtom h1 = new SymbolQueryAtom(new Atom("H"));
        SymbolQueryAtom h2 = new SymbolQueryAtom(new Atom("H"));
        SymbolQueryAtom h3 = new SymbolQueryAtom(new Atom("H"));
        query.addAtom(r);
        query.addAtom(o);
        query.addAtom(c);
        query.addAtom(h1);
        query.addAtom(h2);        
        query.addAtom(h3);        
        query.addBond(new OrderQueryBond(r, o, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(o, c, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(c, h1, CDKConstants.BONDORDER_SINGLE));        
        query.addBond(new OrderQueryBond(c, h2, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(c, h3, CDKConstants.BONDORDER_SINGLE));
        return query;
    }
    public static QueryAtomContainer methoxy_ring() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(METHOXY);
        TopologyAnyAtom r = new TopologyAnyAtom(true); r.setSymbol("R");
        r.setProperty(DONTMARK,query.getID());        
        SymbolQueryAtom o = new SymbolQueryAtom(new Atom("O"));
        SymbolQueryAtom c = new SymbolQueryAtom(new Atom("C"));
        SymbolQueryAtom h1 = new SymbolQueryAtom(new Atom("H"));
        SymbolQueryAtom h2 = new SymbolQueryAtom(new Atom("H"));
        SymbolQueryAtom h3 = new SymbolQueryAtom(new Atom("H"));
        query.addAtom(r);
        query.addAtom(o);
        query.addAtom(c);
        query.addAtom(h1);
        query.addAtom(h2);        
        query.addAtom(h3);        
        query.addBond(new OrderQueryBond(r, o, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(o, c, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(c, h1, CDKConstants.BONDORDER_SINGLE));        
        query.addBond(new OrderQueryBond(c, h2, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(c, h3, CDKConstants.BONDORDER_SINGLE));
        return query;
    }    
    public static QueryAtomContainer hydroxy_ring() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(HYDROXY);
        TopologySymbolQueryAtom c = new TopologySymbolQueryAtom(new Atom("C"),true);
        c.setProperty(DONTMARK,query.getID());        
        SymbolQueryAtom o = new SymbolQueryAtom(new Atom("O"));
        SymbolQueryAtom h = new SymbolQueryAtom(new Atom("H"));
        query.addAtom(h);
        query.addAtom(o);
        query.addAtom(c);
        query.addBond(new OrderQueryBond(o, c, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(o, h, CDKConstants.BONDORDER_SINGLE));        
        return query;
    }
    public static QueryAtomContainer hydroxy1() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(HYDROXY1);
        //aliphatic c
        SymbolQueryAtom c = new TopologySymbolQueryAtom(new Atom("C"),false); 
        c.setProperty(DONTMARK,query.getID());
        SymbolQueryAtom o = new SymbolQueryAtom(new Atom("O"));
        SymbolQueryAtom h = new SymbolQueryAtom(new Atom("H"));
        query.addAtom(h);
        query.addAtom(o);
        query.addAtom(c);
        query.addBond(new OrderQueryBond(o, c, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(o, h, CDKConstants.BONDORDER_SINGLE));        
        return query;
    }        
    // CCCC...COC(=O)R
    public static QueryAtomContainer hydroxyEsterSubstituted() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(HYDROXYESTERSUBSTITED);
        SymbolQueryAtom c1 = new SymbolQueryAtom(new Atom("C")); 
        SymbolQueryAtom c2 = new SymbolQueryAtom(new Atom("C"));
        SymbolQueryAtom o = new SymbolQueryAtom(new Atom("O"));
        SymbolQueryAtom o2 = new SymbolQueryAtom(new Atom("O"));
        SymbolQueryAtom r = new SymbolQueryAtom(new Atom("H"));
        query.addAtom(r);
        query.addAtom(o);query.addAtom(o2);
        query.addAtom(c1);query.addAtom(c2);
        query.addBond(new OrderQueryBond(c1, o, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(o, c2, CDKConstants.BONDORDER_SINGLE));        
        query.addBond(new OrderQueryBond(o2, c2, CDKConstants.BONDORDER_DOUBLE));
        query.addBond(new OrderQueryBond(r, c2, CDKConstants.BONDORDER_SINGLE));
        return query;
    }        
    //R-CH2-R
    public static QueryAtomContainer ethyl() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(CH2);

        SymbolQueryAtom c = new SymbolQueryAtom(new Atom("C"));
        SymbolQueryAtom h1 = new SymbolQueryAtom(new Atom("H"));
        SymbolQueryAtom h2 = new SymbolQueryAtom(new Atom("H"));
        InverseSymbolSetQueryAtom h3 = new InverseSymbolSetQueryAtom();
        h3.addSymbol("H");
        h3.setProperty(DONTMARK,query.getID());
        InverseSymbolSetQueryAtom h4 = new InverseSymbolSetQueryAtom();
        h4.addSymbol("H");
        h4.setProperty(DONTMARK,query.getID());
        
        query.addAtom(c);
        query.addAtom(h1);
        query.addAtom(h2);        
        query.addAtom(h3);        
        query.addBond(new OrderQueryBond(c, h1, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(c, h2, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new AnyOrderQueryBond(c, h3,CDKConstants.BONDORDER_SINGLE));
        query.addBond(new AnyOrderQueryBond(c, h3,CDKConstants.BONDORDER_SINGLE));        
        return query;
    }        
    public static QueryAtomContainer hydrocarbon() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(CH);

        SymbolQueryAtom c = new SymbolQueryAtom(new Atom("C"));
        query.addAtom(c);
        SymbolSetQueryAtom[] neighbors = new SymbolSetQueryAtom[4];
        for (int i =0; i < 4; i++) {
            neighbors[i] = new SymbolSetQueryAtom();
            neighbors[i].addSymbol("C");
            neighbors[i].addSymbol("H");
            query.addAtom(neighbors[i]);
            query.addBond(new OrderQueryBond(c, neighbors[i], CDKConstants.BONDORDER_SINGLE));
        }
        return query;
    }    
    
    
    public static QueryAtomContainer primaryAmine(boolean aliphatic) {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(PRIMARY_AMINE);
        IQueryAtom r;
        if (aliphatic) r= new TopologySymbolQueryAtom(new Atom("C"),false); 
        else r = new SymbolQueryAtom(new Atom("C"));
        ((Atom)r).setProperty(DONTMARK,query.getID());
        SymbolQueryAtom n = new SymbolQueryAtom(new Atom("N"));
        SymbolQueryAtom h1 = new SymbolQueryAtom(new Atom("H"));
        SymbolQueryAtom h2 = new SymbolQueryAtom(new Atom("H"));
        query.addAtom((Atom)r);
        query.addAtom(n);
        query.addAtom(h1);
        query.addAtom(h2);        
        query.addBond(new OrderQueryBond(r, n, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(n, h1, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(n, h2, CDKConstants.BONDORDER_SINGLE));
        return query;
    }
    
    public static QueryAtomContainer secondaryAmine(boolean aliphatic) {
        QueryAtomContainer query = new QueryAtomContainer();
        if (aliphatic) query.setID(SECONDARY_AMINE_ALIPHATIC);
        else  query.setID(SECONDARY_AMINE);
        SymbolQueryAtom[] c = new SymbolQueryAtom[2];
        
        Atom a  = new Atom("N");
        a.setFormalCharge(0);
        SymbolAndChargeQueryAtom n = new SymbolAndChargeQueryAtom(a);
        SymbolQueryAtom h = new SymbolQueryAtom(new Atom("H"));
        
        for (int i=0; i<2; i++) {
	        //if (aliphatic) c[i]= new TopologySymbolQueryAtom(new Atom("C"),false); 
	        //else 
        	if (aliphatic)
        		c[i] = new CHQueryAtom(new Atom("C"));
        	else c[i]= new SymbolQueryAtom(new Atom("C"));
	        query.addAtom(c[i]);
	        if (aliphatic)
	        	query.addBond(new TopologyOrderQueryBond(c[i], n, CDKConstants.BONDORDER_SINGLE,false));
	        else
	        	query.addBond(new OrderQueryBond(c[i], n, CDKConstants.BONDORDER_SINGLE));
        }
        query.addAtom(n);
        query.addAtom(h);        
        query.addBond(new OrderQueryBond(n, h, CDKConstants.BONDORDER_SINGLE));
        return query;
    }

    public static QueryAtomContainer tertiaryAmine() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(TERTIARY_AMINE);
        InverseSymbolSetQueryAtom[] r = new InverseSymbolSetQueryAtom[3];
        SymbolQueryAtom n = new SymbolQueryAtom(new Atom("N"));
        for (int i =0; i < 3; i++) {
	    	r[i] = new InverseSymbolSetQueryAtom();
	    	r[i].setSymbol("*");
        	r[i].addSymbol("H");	
	        
        	query.addAtom(r[i]);
        	query.addBond(new OrderQueryBond(r[i], n, CDKConstants.BONDORDER_SINGLE));        	
	    }    
        return query;
    }	
/*
    public static QueryAtomContainer amine(int order, boolean aliphatic) {
        QueryAtomContainer query = new QueryAtomContainer();
        switch (order) {
        //primary
        case 1:{query.setID(PRIMARY_AMINE); break;}
        //secondary
        case 2:{query.setID(SECONDARY_AMINE); break;}
        //tertiary
        case 3: {query.setID(TERTIARY_AMINE); break;}
        //any
        default :	{order= 0;query.setID(AMINE); break;}
        }
        InverseSymbolSetQueryAtom[] r = new InverseSymbolSetQueryAtom[3];
        SymbolQueryAtom n = new SymbolQueryAtom(new Atom("N"));
        for (int i =0; i < order; i++) {
	    	r[i] = new InverseSymbolSetQueryAtom();
        	r[i].addSymbol("H");
        	query.addAtom(r[i]);
        	query.addBond(new OrderQueryBond(r[i], n, CDKConstants.BONDORDER_SINGLE));        	
	    }    
        return query;
    }	
    */
    // [N+].[Cl-]
    public static QueryAtomContainer hydrochlorideOfAmine(int amine) {
    	//TODO verify the correct way of representation hydrochlorideOfAmine
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(HYDROCHLORIDE_OF_AMINE);
        Atom a = new Atom("N");
        a.setFormalCharge(1);
        SymbolAndChargeQueryAtom n = new SymbolAndChargeQueryAtom(a);
        //SymbolQueryAtom n = new SymbolQueryAtom(a);
        query.addAtom(n);
        
        Atom chlor = new Atom("Cl");
        chlor.setFormalCharge(-1);
        SymbolAndChargeQueryAtom cl = new SymbolAndChargeQueryAtom(chlor);
        query.addAtom(cl);        
        SymbolQueryAtom[] r = new SymbolQueryAtom[4];
        for (int i=0; i < 4; i++) {
        	
        	if ((i+1) > amine) r[i] = new SymbolQueryAtom(new Atom("H"));
        	else r[i] = new SymbolQueryAtom(new Atom("C"));
        	query.addAtom(r[i]);
        	query.addBond(new OrderQueryBond(n, r[i], CDKConstants.BONDORDER_SINGLE));
        }
        //TODO should verify somehow for ionic bond, not any ...
        query.addBond(new AnyOrderQueryBond(n,cl, CDKConstants.BONDORDER_SINGLE));
        return query;
    }

    public static QueryAtomContainer hydrochlorideOfAmine3() {
    	//TODO verify the correct way of representation hydrochlorideOfAmine
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(HYDROCHLORIDE_OF_AMINE);
        Atom a = new Atom("N");
        a.setFormalCharge(1);
        SymbolAndChargeQueryAtom n = new SymbolAndChargeQueryAtom(a);
        //SymbolQueryAtom n = new SymbolQueryAtom(a);
        query.addAtom(n);
        
        Atom chlor = new Atom("Cl");
        chlor.setFormalCharge(-1);
        SymbolAndChargeQueryAtom cl = new SymbolAndChargeQueryAtom(chlor);
        query.addAtom(cl);        
        SymbolQueryAtom[] r = new SymbolQueryAtom[3];
        for (int i=0; i < 3; i++) {
        	
        	r[i] = new SymbolQueryAtom(new Atom("C"));
        	query.addAtom(r[i]);
        	if (i==0)
        		query.addBond(new OrderQueryBond(n, r[i], CDKConstants.BONDORDER_DOUBLE));
        	else
        		query.addBond(new OrderQueryBond(n, r[i], CDKConstants.BONDORDER_SINGLE));
        }
        //TODO should verify somehow for ionic bond, not any ...
        query.addBond(new AnyOrderQueryBond(n,cl, CDKConstants.BONDORDER_SINGLE));
        return query;
    }
    
    // [N+].[Cl-]
    
    public static QueryAtomContainer hydrochlorideOfAmineBreakable() {
    	//TODO verify the correct way of representation hydrochlorideOfAmine
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(HYDROCHLORIDE_OF_AMINE);
        Atom a = new Atom("N");
        a.setFormalCharge(1);
        SymbolAndChargeQueryAtom n = new SymbolAndChargeQueryAtom(a);
        query.addAtom(n);
        
        Atom chlor = new Atom("Cl");
        chlor.setFormalCharge(-1);
        
        SymbolAndChargeQueryAtom cl = new SymbolAndChargeQueryAtom(chlor);
        cl.setProperty(DONTMARK,query.getID());
        query.addAtom(cl);
        
        //TODO moze li da ima drugi elementi zakaceni za N
        SymbolSetQueryAtom[] r = new SymbolSetQueryAtom[2];
        for (int i=0; i < r.length; i++) {
        	r[i] = new SymbolSetQueryAtom();
        	r[i].addSymbol("C");
        	r[i].addSymbol("H");
        	query.addAtom(r[i]);
        	query.addBond(new SingleOrDoubleQueryBond(n, r[i]));
        }
        //TODO should verify somehow for ionic bond, not any ...
        query.addBond(new OrderQueryBond(n,cl, CDKConstants.BONDORDER_SINGLE));
        return query;
    }
    
    //S(=O)(=O)(OH)[O-][N+]
    public static QueryAtomContainer sulphateOfAmine(int amine) {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(SULPHATE_OF_AMINE);
        SymbolQueryAtom s = new SymbolQueryAtom(new Atom("S"));
        query.addAtom(s);        
        IQueryAtom[] o = new IQueryAtom[4];
        for (int i=0; i < 4; i++) {
        	if (i==0) {
        		Atom a = new Atom("O");
        		a.setFormalCharge(-1);
        		o[i] = new SymbolAndChargeQueryAtom(a);
        	} else	o[i] = new SymbolQueryAtom(new Atom("O"));
        	query.addAtom((Atom)o[i]);
        	if (i<2)
        		query.addBond(new OrderQueryBond(s, o[i], CDKConstants.BONDORDER_SINGLE));
        	else
        		query.addBond(new OrderQueryBond(s, o[i], CDKConstants.BONDORDER_DOUBLE));
        }
        
        
        /*
         * S-[O-].[N+]
         * S-O
         * S=O
         * S=O
         * 
         */
        /*TODO verify if this is necessary
        SymbolQueryAtom h = new SymbolQueryAtom(new Atom("H"));
        query.addAtom(h);
        query.addBond(new OrderQueryBond(h, o[1], CDKConstants.BONDORDER_SINGLE));
        */
        Atom a = new Atom("N");
        a.setFormalCharge(1);
        SymbolAndChargeQueryAtom n = new SymbolAndChargeQueryAtom(a);
        //SymbolQueryAtom n = new SymbolQueryAtom(a);
        query.addAtom(n);
   
        if (amine > 0) { //primary, sec, etc
	        SymbolSetQueryAtom[] r = new SymbolSetQueryAtom[4];
	        for (int i=0; i < 4; i++) {
	        	r[i] = new SymbolSetQueryAtom();
	        	if ((i+1) > amine) r[i].addSymbol("H");
	        	else r[i].addSymbol("C");
	        	query.addAtom(r[i]);
	        	query.addBond(new OrderQueryBond(n, r[i], CDKConstants.BONDORDER_SINGLE)); //[N+]-R  or [N+]-H
	        }
        }
        //TODO should verify somehow for ionic bond, not any ...
        query.addBond(new AnyOrderQueryBond(n,o[0], CDKConstants.BONDORDER_SINGLE)); //[N+]-[O-]
        return query;
    }
    /**
     * This is {@link #sulphateOfAmine(int)} with the ionic bonnd between [N+] and [O-] 
     * marked with {@link #DONTMARK} so it can be broken by {@link #detachGroup(IAtomContainer, QueryAtomContainer)} method 
     * @return QueryAtomContainer  the query
     */
    public static QueryAtomContainer sulphateOfAmineBreakable() {
    	QueryAtomContainer aminoSulphate = FunctionalGroups.sulphateOfAmine(0);
		//find [N+].[O-] bond and mark it as DONTMARK
		for (int i=0; i <aminoSulphate.getBondCount();i++) {
			IAtom[] a = aminoSulphate.getBondAt(i).getAtoms();
			IAtom n = null; 
			IAtom o = null;
			for (int k=0;k<a.length;k++)
				if (a[k].getSymbol().equals("N")) n = a[k];
				else if (a[k].getSymbol().equals("O")) o = a[k];
			if ((n !=null) && (o !=null)) { //this is the bond
				n.setProperty(FunctionalGroups.DONTMARK,aminoSulphate.getID());
				logger.debug("Ma");
				break;
			}
		}
		return aminoSulphate;
    }
    public static QueryAtomContainer cyano() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(CYANO);
        
        SymbolSetQueryAtom a = new SymbolSetQueryAtom();
        a.addSymbol("C");
        a.addSymbol("H");
        SymbolQueryAtom c = new SymbolQueryAtom(new Atom("C"));
        SymbolQueryAtom n = new SymbolQueryAtom(new Atom("N"));
        
        query.addAtom(a);
        query.addAtom(c);
        query.addAtom(n);
        query.addBond(new OrderQueryBond(a, c, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(c, n, CDKConstants.BONDORDER_TRIPLE));
        return query;
    }
    public static QueryAtomContainer nitro2double() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(NITRO);
        InverseSymbolSetQueryAtom r = new InverseSymbolSetQueryAtom();
        r.addSymbol("H");
        SymbolQueryAtom n = new SymbolQueryAtom(new Atom("N"));
        SymbolQueryAtom o1 = new SymbolQueryAtom(new Atom("O"));
        SymbolQueryAtom o2 = new SymbolQueryAtom(new Atom("O"));        
        
        query.addAtom(r);query.addAtom(n);
        query.addAtom(o1);query.addAtom(o2);
        query.addBond(new OrderQueryBond(r, n, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(n, o1, CDKConstants.BONDORDER_DOUBLE));
        query.addBond(new OrderQueryBond(n, o2, CDKConstants.BONDORDER_DOUBLE));        
        return query;
    }
    public static QueryAtomContainer nitro1double() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(NITRO);
        InverseSymbolSetQueryAtom r = new InverseSymbolSetQueryAtom();
        
        r.addSymbol("H");
        Atom a = new Atom("N");      a.setFormalCharge(+1);
        SymbolAndChargeQueryAtom n = new SymbolAndChargeQueryAtom(a);
        
        SymbolQueryAtom o1 = new SymbolQueryAtom(new Atom("O"));
        
        Atom o = new Atom("O");      o.setFormalCharge(-1);
        SymbolAndChargeQueryAtom o2 = new SymbolAndChargeQueryAtom(o);
        
        
        query.addAtom(r);query.addAtom(n);
        query.addAtom(o1);query.addAtom(o2);
        query.addBond(new OrderQueryBond(r, n, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(n, o1, CDKConstants.BONDORDER_DOUBLE));
        query.addBond(new OrderQueryBond(n, o2, CDKConstants.BONDORDER_SINGLE));        
        return query;
    }        
    public static QueryAtomContainer Nnitroso() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(NNITROSO);
        InverseSymbolSetQueryAtom r1 = new InverseSymbolSetQueryAtom();
        r1.addSymbol("H");
        InverseSymbolSetQueryAtom r2 = new InverseSymbolSetQueryAtom();
        r2.addSymbol("H");
        SymbolQueryAtom n1 = new SymbolQueryAtom(new Atom("N"));
        SymbolQueryAtom n2 = new SymbolQueryAtom(new Atom("N"));
        SymbolQueryAtom o = new SymbolQueryAtom(new Atom("O"));
        
        query.addAtom(r1);query.addAtom(r2);
        query.addAtom(n1);query.addAtom(n2);
        query.addAtom(o);
        query.addBond(new OrderQueryBond(r1, n1, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(r2, n1, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(n1, n2, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(n2, o, CDKConstants.BONDORDER_DOUBLE));
        return query;
    }
    public static QueryAtomContainer diAzo() {
        SymbolQueryAtom n1 = new SymbolQueryAtom(new Atom("N"));
        SymbolQueryAtom n2 = new SymbolQueryAtom(new Atom("N"));
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(DIAZO);
        query.addAtom(n1);query.addAtom(n2);
        query.addBond(new OrderQueryBond(n1, n2, CDKConstants.BONDORDER_TRIPLE));
        return query;
    }
    public static QueryAtomContainer triAzeno() {
        
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(TRIAZENO);
        
     	SymbolQueryAtom r = new SymbolQueryAtom(new Atom("C"));
     	r.setProperty(DONTMARK,query.getID());
        SymbolQueryAtom n1 = new SymbolQueryAtom(new Atom("N"));
        SymbolQueryAtom n2 = new SymbolQueryAtom(new Atom("N"));
        SymbolQueryAtom n3 = new SymbolQueryAtom(new Atom("N"));
        SymbolQueryAtom h1 = new SymbolQueryAtom(new Atom("H"));
        SymbolQueryAtom h2 = new SymbolQueryAtom(new Atom("H"));
        
        query.addAtom(r);query.addAtom(n1);
        query.addAtom(n2);query.addAtom(n3);
        query.addAtom(h1);query.addAtom(h2);
        query.addBond(new OrderQueryBond(r, n1, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(n1, n2, CDKConstants.BONDORDER_DOUBLE));
        query.addBond(new OrderQueryBond(n2, n3, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(n3, h1, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(n3, h2, CDKConstants.BONDORDER_SINGLE));
        return query;
    }
    
    
    public static QueryAtomContainer quaternaryNitrogen1(boolean charged) {
    	
     	AnyAtom r1 = new AnyAtom();
     	AnyAtom r2 = new AnyAtom();
     	AnyAtom r3 = new AnyAtom();
     	AnyAtom r4 = new AnyAtom();

     	//SymbolQueryAtom n = new SymbolQueryAtom(new Atom("N"));
     	Atom a = new Atom("N");
     	IQueryAtom n;
     	if (charged) {
     		a.setFormalCharge(1);
     		n = new SymbolAndChargeQueryAtom(a);
     	} else n = new SymbolQueryAtom(a);

        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(QUATERNARY_NITROGEN);
        query.addAtom(r1);query.addAtom(r2);
        query.addAtom(r3);query.addAtom(r4);
        query.addAtom((Atom)n);
        query.addBond(new OrderQueryBond(r1, n, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(r2, n, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(r3, n, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(r4, n, CDKConstants.BONDORDER_SINGLE));
            
        return query;
    }
    
    public static QueryAtomContainer quaternaryNitrogen2(boolean charged) {
    	
     	AnyAtom r1 = new AnyAtom();
     	AnyAtom r2 = new AnyAtom();
     	AnyAtom r3 = new AnyAtom();

     	//SymbolQueryAtom n = new SymbolQueryAtom(new Atom("N"));
     	Atom a = new Atom("N");
     	IQueryAtom n;
     	if (charged) {
     		a.setFormalCharge(1);
     		n = new SymbolAndChargeQueryAtom(a);
     	} else n = new SymbolQueryAtom(a);      	//else to cope with wrong record
 
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(QUATERNARY_NITROGEN);
        query.addAtom(r1);query.addAtom(r2);
        query.addAtom(r3);
        query.addAtom((Atom)n);
        query.addBond(new OrderQueryBond(r1, n, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(r2, n, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(r3, n, CDKConstants.BONDORDER_DOUBLE));
            
        return query;
    }
    
    
    public static QueryAtomContainer quarternaryNitrogenException() {
     	AnyAtom nr1 = new AnyAtom();
     	AnyAtom nr2 = new AnyAtom();
     	SymbolQueryAtom c = new SymbolQueryAtom(new Atom("C"));
     	Atom a = new Atom("N");
     	a.setFormalCharge(1);
        SymbolAndChargeQueryAtom n = new SymbolAndChargeQueryAtom(a);
        InverseSymbolSetQueryAtom cr1 = new InverseSymbolSetQueryAtom();
        cr1.addSymbol("H");
        InverseSymbolSetQueryAtom cr2 = new InverseSymbolSetQueryAtom();
        cr2.addSymbol("H");
        
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(QUATERNARY_NITROGEN_EXCEPTION);
        query.addAtom(nr1);query.addAtom(nr2);
        query.addAtom(cr1);query.addAtom(cr2);
        query.addAtom(n);query.addAtom(c);
        query.addAtom(n);
        query.addBond(new OrderQueryBond(nr1, n, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(nr2, n, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(c, n, CDKConstants.BONDORDER_DOUBLE));
        query.addBond(new OrderQueryBond(cr1, c, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(cr2, c, CDKConstants.BONDORDER_SINGLE));
        
        return query;
    }    
    //    R-C(=O)-OH
    public static QueryAtomContainer carboxylicAcid() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(CARBOXYLIC_ACID);        
        AnyAtom r = new AnyAtom();
        r.setProperty(DONTMARK,query.getID());
        
        SymbolQueryAtom c = new SymbolQueryAtom(new Atom("C"));
        SymbolQueryAtom o1 = new SymbolQueryAtom(new Atom("O"));
        SymbolQueryAtom o2 = new SymbolQueryAtom(new Atom("O"));
        SymbolQueryAtom h = new SymbolQueryAtom(new Atom("H"));        
        query.addAtom(r); query.addAtom(c);
        query.addAtom(o1); query.addAtom(o2);query.addAtom(h);
        query.addBond(new OrderQueryBond(r, c, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(c, o1, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(h, o1, CDKConstants.BONDORDER_SINGLE));        
        query.addBond(new OrderQueryBond(c, o2, CDKConstants.BONDORDER_DOUBLE));
        return query;
    }
    public static QueryAtomContainer acyclic_acetal() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(ACETAL);        
        SymbolQueryAtom[] c = new SymbolQueryAtom[3];
        for (int i =0; i < 3; i++) {
            c[i] = new SymbolQueryAtom(new Atom("C"));
            query.addAtom(c[i]);
        }
        SymbolQueryAtom[] o = new SymbolQueryAtom[2];
        for (int i =0; i < 2; i++) {
            o[i] = new SymbolQueryAtom(new Atom("O"));
            query.addAtom(o[i]);
        }
        query.addBond(new TopologyOrderQueryBond(c[0], o[0], CDKConstants.BONDORDER_SINGLE,false));
        query.addBond(new TopologyOrderQueryBond(c[1], o[0], CDKConstants.BONDORDER_SINGLE,false));
        query.addBond(new TopologyOrderQueryBond(c[1], o[1], CDKConstants.BONDORDER_SINGLE,false));
        query.addBond(new TopologyOrderQueryBond(c[2], o[1], CDKConstants.BONDORDER_SINGLE,false));        
        return query;
    	
    }
    // -C-O-C-O-C-
    public static QueryAtomContainer acetal() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(ACETAL);        
        SymbolQueryAtom[] c = new SymbolQueryAtom[3];
        for (int i =0; i < 3; i++) {
            c[i] = new SymbolQueryAtom(new Atom("C"));
            query.addAtom(c[i]);
        }
        SymbolQueryAtom[] o = new SymbolQueryAtom[2];
        for (int i =0; i < 2; i++) {
            o[i] = new SymbolQueryAtom(new Atom("O"));
            query.addAtom(o[i]);
        }
        query.addBond(new OrderQueryBond(c[0], o[0], CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(c[1], o[0], CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(c[1], o[1], CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(c[2], o[1], CDKConstants.BONDORDER_SINGLE));        
        return query;
    }

    // C-O-C  have to check if substituted
    public static QueryAtomContainer ether() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(ETHER);
        SymbolQueryAtom[] r = new SymbolQueryAtom[2];
        for (int i =0; i < 2; i++) {
            r[i] = new CHQueryAtom(new Atom("C"));
//            r[i].setProperty(DONTMARK,query.getID());            
            query.addAtom(r[i]);
        }
        //TotalValencyAtom s = new TotalValencyAtom(2);
        SymbolQueryAtom s = new SymbolQueryAtom(new Atom("O"));
        query.addAtom(s);
        query.addBond(new OrderQueryBond(r[0], s, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(r[1], s, CDKConstants.BONDORDER_SINGLE));
        return query;
    }
    // C-O-C  have to check if substituted
    public static QueryAtomContainer alkoxy() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(ETHER);
        SymbolQueryAtom[] r = new SymbolQueryAtom[2];
        for (int i =0; i < 2; i++) {
            r[i] = new SymbolQueryAtom(new Atom("C"));
            query.addAtom(r[i]);
        }
        //TotalValencyAtom s = new TotalValencyAtom(2);
        SymbolQueryAtom s = new SymbolQueryAtom(new Atom("O"));
        query.addAtom(s);
        query.addBond(new OrderQueryBond(r[0], s, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(r[1], s, CDKConstants.BONDORDER_SINGLE));
        return query;
    }    
    // C-O-C  have to check if substituted
    public static QueryAtomContainer methylether() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(METHYLETHER);
        
        SymbolQueryAtom o = new SymbolQueryAtom(new Atom("O"));
        query.addAtom(o);        
        SymbolQueryAtom[] r = new SymbolQueryAtom[2];
        for (int i =0; i < 2; i++) {
            r[i] = new SymbolQueryAtom(new Atom("C"));
            query.addAtom(r[i]);
            query.addBond(new OrderQueryBond(r[i], o, CDKConstants.BONDORDER_SINGLE));            
        }
        SymbolQueryAtom[] h = new SymbolQueryAtom[3];
        for (int i =0; i < 3; i++) {
            h[i] = new SymbolQueryAtom(new Atom("H"));
            query.addAtom(h[i]);
            query.addBond(new OrderQueryBond(h[i], r[0], CDKConstants.BONDORDER_SINGLE));            
        }        
        return query;
    }
    
    // C-S-C
    public static QueryAtomContainer sulphide() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(SULPHIDE);
        SymbolQueryAtom[] r = new SymbolQueryAtom[2];
        for (int i =0; i < 2; i++) {
            r[i] = new SymbolQueryAtom(new Atom("C"));
            query.addAtom(r[i]);
        }
        //TotalValencyAtom s = new TotalValencyAtom(2);
        SymbolQueryAtom s = new SymbolQueryAtom(new Atom("S"));
        query.addAtom(s);
        query.addBond(new OrderQueryBond(r[0], s, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(r[1], s, CDKConstants.BONDORDER_SINGLE));
        return query;
    }

    // R-SH
    public static QueryAtomContainer mercaptan() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(MERCAPTAN);
        SymbolQueryAtom c = new SymbolQueryAtom(new Atom("C"));
        SymbolQueryAtom s = new SymbolQueryAtom(new Atom("S"));
        SymbolQueryAtom h = new SymbolQueryAtom(new Atom("H"));
        
        query.addAtom(s);query.addAtom(c);query.addAtom(h);
        query.addBond(new OrderQueryBond(c, s, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(s, h, CDKConstants.BONDORDER_SINGLE));
        return query;
    }   
	//R-S-C(=O)-R'
    //R-S-C(=S)-R'
    public static QueryAtomContainer thioester() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(THIOESTER);        
        InverseSymbolSetQueryAtom r = new InverseSymbolSetQueryAtom();
        r.addSymbol("H");
        r.addSymbol("S");
        r.addSymbol("O");
        r.setProperty(DONTMARK,query.getID());
        
        InverseSymbolSetQueryAtom e = new InverseSymbolSetQueryAtom();
        e.addSymbol("H");
        e.setProperty(DONTMARK,query.getID());
        
        SymbolQueryAtom c = new SymbolQueryAtom(new Atom("C"));
        SymbolQueryAtom s = new SymbolQueryAtom(new Atom("S"));
        SymbolSetQueryAtom o2 = new SymbolSetQueryAtom();
        o2.addSymbol("O");
        o2.addSymbol("S");
        
        query.addAtom(r); query.addAtom(e); query.addAtom(c);
        query.addAtom(s); query.addAtom(o2);
        query.addBond(new OrderQueryBond(r, c, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(c, s, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(c, o2, CDKConstants.BONDORDER_DOUBLE));
        query.addBond(new OrderQueryBond(s, e, CDKConstants.BONDORDER_SINGLE));
        return query;
    }
	
    // RC(=O)OR
    public static QueryAtomContainer ester() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(ESTER);        
        AnyAtom r = new AnyAtom();
        r.setProperty(DONTMARK,query.getID());
        InverseSymbolSetQueryAtom e = new InverseSymbolSetQueryAtom();
        e.addSymbol("H");
        e.setProperty(DONTMARK,query.getID());
        
        SymbolQueryAtom c = new SymbolQueryAtom(new Atom("C"));
        SymbolQueryAtom o1 = new SymbolQueryAtom(new Atom("O"));
        SymbolQueryAtom o2 = new SymbolQueryAtom(new Atom("O"));
        query.addAtom(r); query.addAtom(e); query.addAtom(c);
        query.addAtom(o1); query.addAtom(o2);
        query.addBond(new OrderQueryBond(r, c, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(c, o1, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(c, o2, CDKConstants.BONDORDER_DOUBLE));
        query.addBond(new OrderQueryBond(o1, e, CDKConstants.BONDORDER_SINGLE));
        return query;
    }
    // ROC(=O)OR
    public static QueryAtomContainer carbonate() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(CARBONATE);        
        AnyAtom r = new AnyAtom();
        r.setProperty(DONTMARK,query.getID());
        InverseSymbolSetQueryAtom e = new InverseSymbolSetQueryAtom();
        e.addSymbol("H");
        e.setProperty(DONTMARK,query.getID());
        
        SymbolQueryAtom c = new SymbolQueryAtom(new Atom("C"));
        SymbolQueryAtom o1 = new SymbolQueryAtom(new Atom("O"));
        SymbolQueryAtom o2 = new SymbolQueryAtom(new Atom("O"));
        SymbolQueryAtom o3 = new SymbolQueryAtom(new Atom("O"));        
        query.addAtom(r); query.addAtom(e); query.addAtom(c);
        query.addAtom(o1); query.addAtom(o2); query.addAtom(o3);
        query.addBond(new OrderQueryBond(o3, r, CDKConstants.BONDORDER_SINGLE));        
        query.addBond(new OrderQueryBond(o3, c, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(c, o1, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(c, o2, CDKConstants.BONDORDER_DOUBLE));
        query.addBond(new OrderQueryBond(o1, e, CDKConstants.BONDORDER_SINGLE));
        return query;
    }    
    public static QueryAtomContainer anhydride() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(ANHYDRIDE);        

        SymbolQueryAtom o = new SymbolQueryAtom(new Atom("O"));
        
        SymbolQueryAtom r = new SymbolQueryAtom(new Atom("C"));
        r.setProperty(DONTMARK,query.getID());
        
        SymbolQueryAtom e = new SymbolQueryAtom(new Atom("C"));
        r.setProperty(DONTMARK,query.getID());
        
        SymbolQueryAtom c = new SymbolQueryAtom(new Atom("C"));
        SymbolQueryAtom o1 = new SymbolQueryAtom(new Atom("O"));
        SymbolQueryAtom o2 = new SymbolQueryAtom(new Atom("O"));
        query.addAtom(r); query.addAtom(e); query.addAtom(c);
        query.addAtom(o1); query.addAtom(o2);
        query.addBond(new TopologyAnyBond(r, c, true));
        query.addBond(new TopologyOrderQueryBond(c, o1, CDKConstants.BONDORDER_SINGLE,true));
        query.addBond(new TopologyOrderQueryBond(c, o2, CDKConstants.BONDORDER_DOUBLE,false));
        query.addBond(new TopologyAnyBond(o1, e, true));
        
        query.addBond(new TopologyOrderQueryBond(e, o, CDKConstants.BONDORDER_DOUBLE,false));
        return query;
    }
    /**
     * 
     * @param abUnsaturated if true, this will be alpha -beta unsaturated lactone
     * @return
     */
    public static QueryAtomContainer lactone(boolean abUnsaturated) {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(LACTONE);        

        SymbolQueryAtom r = new SymbolQueryAtom(new Atom("C"));
        r.setProperty(DONTMARK,query.getID());
        
        
        SymbolQueryAtom e = new SymbolQueryAtom(new Atom("C"));
        r.setProperty(DONTMARK,query.getID());
        
        SymbolQueryAtom c = new SymbolQueryAtom(new Atom("C"));
        SymbolQueryAtom o1 = new SymbolQueryAtom(new Atom("O"));
        SymbolQueryAtom o2 = new SymbolQueryAtom(new Atom("O"));
        query.addAtom(r); query.addAtom(e); query.addAtom(c);
        query.addAtom(o1); query.addAtom(o2);
        query.addBond(new TopologyAnyBond(r, c, true));
        query.addBond(new TopologyOrderQueryBond(c, o1, CDKConstants.BONDORDER_SINGLE,true));
        query.addBond(new TopologyOrderQueryBond(c, o2, CDKConstants.BONDORDER_DOUBLE,false));
        query.addBond(new TopologyAnyBond(o1, e, true));
        
        if (abUnsaturated) {
            SymbolQueryAtom ab = new SymbolQueryAtom(new Atom("C"));
            ab.setProperty(DONTMARK,query.getID());
            query.addAtom(ab);
            query.addBond(new TopologyOrderQueryBond(r, ab,CDKConstants.BONDORDER_DOUBLE, true));
        }
        
        return query;
    }
    
    
    public static QueryAtomContainer lactoneBreakable() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(LACTONE);        
        SymbolQueryAtom r = new SymbolQueryAtom(new Atom("C"));
        //AnyAtom r = new AnyAtom();
        SymbolQueryAtom e = new SymbolQueryAtom(new Atom("C"));
        e.setProperty(DONTMARK,query.getID());
        
        SymbolQueryAtom c = new SymbolQueryAtom(new Atom("C"));
        SymbolQueryAtom o1 = new SymbolQueryAtom(new Atom("O"));
        SymbolQueryAtom o2 = new SymbolQueryAtom(new Atom("O"));
        query.addAtom(r); query.addAtom(e); query.addAtom(c);
        query.addAtom(o1); query.addAtom(o2);
        query.addBond(new TopologyOrderQueryBond(r, c, CDKConstants.BONDORDER_SINGLE,true));
        query.addBond(new TopologyOrderQueryBond(c, o1, CDKConstants.BONDORDER_SINGLE,true));
        query.addBond(new TopologyOrderQueryBond(c, o2, CDKConstants.BONDORDER_DOUBLE,false));
        TopologyOrderQueryBond b = new TopologyOrderQueryBond(o1, e, CDKConstants.BONDORDER_SINGLE,true);
        b.setProperty(DONTMARK,query.getID());
        query.addBond(b);
        return query;
    }
    
    public static boolean isCyclicDiester(IAtomContainer mol, RingSet rings) {
        QueryAtomContainer query = lactone(false);
        query.setID(CYCLIC_DIESTER);     
        List list = getUniqueBondMap(mol,query,false);
        
        ArrayList esterGroups = new ArrayList();
        /*Count ester groups per ring, should be exactly 2
         * Use the fact that bond map marks each ester group with unique identifier 
        */
        if ((list !=null) && (list.size()>=2)) { //can be
        	markMaps(mol,query,list);
        	for (int i=0; i < rings.getAtomContainerCount(); i++) { 
        		Ring ring = (Ring) rings.getAtomContainer(i);
        		esterGroups.clear();
        		for (int b=0;b<ring.getBondCount();b++) {
        			Object o = ring.getBondAt(b).getProperty(query.getID());
        			if (o==null) continue;
        			else if (!esterGroups.contains(o)) esterGroups.add(o);
        		}
        		if (esterGroups.size() == 2) {
        			logger.debug("Ring with two ester groups\tNO");
        			for (int a=0;a<ring.getAtomCount();a++) 
        				if (ring.getAtomAt(a).getSymbol().equals("C")) continue;
        				else if (ring.getAtomAt(a).getSymbol().equals("O")) {
        					if (ring.getAtomAt(a).getProperty(query.getID()) == null) {
        						// O atom only allowed if from ester group
	        					logger.debug("Heteroring\tYES");
	        					return false;
        					}
        				} else {
        					logger.debug("Heteroring\tYES");
        					return false;        					
        				}
        			logger.info("Cyclic diester\tYES");        				
        			return true;
        		}
        	} 
        	logger.debug("Ring with two ester groups\tNO");
        	return false;
    	} else {
    		logger.debug("Less than 2 ester groups ");
    		return false;
    	}
    }    
    public static QueryAtomContainer isopreneUnit() {
    	QueryAtomContainer query = new QueryAtomContainer();
        query.setID(ISOPRENE);
        SymbolQueryAtom[] c = new SymbolQueryAtom[5];
        for (int i =0; i < c.length; i++) {
            c[i] = new SymbolQueryAtom(new Atom("C"));   
            query.addAtom(c[i]);

        }
        for (int i =1; i < c.length-1; i++) 
        	query.addBond(new SingleOrDoubleQueryBond(c[i-1], c[i]));
        query.addBond(new OrderQueryBond(c[1], c[4],CDKConstants.BONDORDER_SINGLE));
        
        return query;
    }
    //CC(=O)C
    public static QueryAtomContainer ketone() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(KETONE);
        
        SymbolQueryAtom c =  new SymbolQueryAtom(new Atom("C")); 
        InverseSymbolSetQueryAtom[] r = new InverseSymbolSetQueryAtom[2];
        for (int i =0; i < 2; i++) {
            r[i] = new InverseSymbolSetQueryAtom();
            r[i].addSymbol("H");
            r[i].addSymbol("O");
            r[i].addSymbol("S");
            r[i].setProperty(DONTMARK,query.getID());
            query.addAtom(r[i]);
        }
        
        SymbolQueryAtom o = new SymbolQueryAtom(new Atom("O"));
        query.addAtom(o);
        query.addBond(new OrderQueryBond(r[0], c, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(c, o, CDKConstants.BONDORDER_DOUBLE));
        query.addBond(new OrderQueryBond(r[1], c, CDKConstants.BONDORDER_SINGLE));
        return query;
    }
    public static QueryAtomContainer ketone_ring() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(KETONE);        
        SymbolQueryAtom[] c = new SymbolQueryAtom[3];
        for (int i =0; i < 3; i++) {
       		c[i] = new TopologySymbolQueryAtom(new Atom("C"),true);   
            query.addAtom(c[i]);
        }
        
        SymbolQueryAtom o = new SymbolQueryAtom(new Atom("O"));
        query.addAtom(o);
        query.addBond(new OrderQueryBond(c[0], c[1], CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(c[1], o, CDKConstants.BONDORDER_DOUBLE));
        query.addBond(new OrderQueryBond(c[1], c[2], CDKConstants.BONDORDER_SINGLE));
        return query;
    }
    
    public static QueryAtomContainer sidechain_ketone() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(KETONE_SIDECHAIN);        
        SymbolQueryAtom[] c = new SymbolQueryAtom[3];
        for (int i =0; i < 3; i++) {
        	if (i==1)
        		c[i] = new TopologySymbolQueryAtom(new Atom("C"),false);
        	else
        		c[i] = new SymbolQueryAtom(new Atom("C"));   
            query.addAtom(c[i]);
        }
        
        SymbolQueryAtom o = new TopologySymbolQueryAtom(new Atom("O"),false);
        query.addAtom(o);
        query.addBond(new OrderQueryBond(c[0], c[1], CDKConstants.BONDORDER_SINGLE));
        query.addBond(new TopologyOrderQueryBond(c[1], o, CDKConstants.BONDORDER_DOUBLE,false));
        query.addBond(new OrderQueryBond(c[1], c[2], CDKConstants.BONDORDER_SINGLE));
        return query;
    }
    
    /**
     * RC(=O)H
     * An aldehyde is either a functional group consisting of a terminal carbonyl group, 
     * or a compound containing a terminal carbonyl group. 
     * (Where -R represents the carbon chain.) 
     */
    public static QueryAtomContainer aldehyde() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(ALDEHYDE);
        /*
        InverseSymbolSetQueryAtom r = new InverseSymbolSetQueryAtom();
        //r.addSymbol("H"); ne moze da poznae formaldehyde
        r.addSymbol("O"); //shte bude ester ako e O
        */
        SymbolSetQueryAtom r = new SymbolSetQueryAtom();
        r.addSymbol("C");
        r.addSymbol("H");
        r.setProperty(DONTMARK,query.getID());
        
        SymbolQueryAtom c = new SymbolQueryAtom(new Atom("C"));
        SymbolQueryAtom o = new SymbolQueryAtom(new Atom("O"));        
        SymbolQueryAtom h = new SymbolQueryAtom(new Atom("H"));
        query.addAtom(r);query.addAtom(c);query.addAtom(o);query.addAtom(h);
        query.addBond(new OrderQueryBond(c, h, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(c, o, CDKConstants.BONDORDER_DOUBLE));
        query.addBond(new OrderQueryBond(r, c, CDKConstants.BONDORDER_SINGLE));
        return query;
    }
    //C=O
    public static QueryAtomContainer carbonyl() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(CARBONYL);
        SymbolQueryAtom c = new SymbolQueryAtom(new Atom("C"));
        SymbolQueryAtom o = new SymbolQueryAtom(new Atom("O"));        
        query.addAtom(c);query.addAtom(o);
        query.addBond(new OrderQueryBond(c, o, CDKConstants.BONDORDER_DOUBLE));
        return query;
    }    

    /**
     * 
     * @param aliphatic  - if true looks for aliphatic alcohols, if not for any alcohols
     * @return
     */
    public static QueryAtomContainer alcohol(boolean aliphatic) {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(ALCOHOL);
        CHQueryAtom r;
        if (aliphatic) { 
            r= new AliphaticCHQueryAtom(new Atom("C"));
        	query.addAtom(r);
        } else { 
        	r = new CHQueryAtom(new Atom("C"));
        	query.addAtom((SymbolQueryAtom)r); 
        }
        r.setProperty(DONTMARK,query.getID());
        SymbolQueryAtom o = new SymbolQueryAtom(new Atom("O"));
        query.addAtom(o);
        SymbolQueryAtom h = new SymbolQueryAtom(new Atom("H"));
        query.addAtom(h);
        query.addBond(new OrderQueryBond(r, o, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(o, h, CDKConstants.BONDORDER_SINGLE));
        return query;
    }   
    //HNS(=O)(=O)(O-Me)N
    
    public static QueryAtomContainer sulphamate(String[] metals) {
    	SymbolSetQueryAtom me = null;
    	if (metals != null) {
	    	me = new SymbolSetQueryAtom();
	    	for (int i=0; i < metals.length; i++) me.addSymbol(metals[i]);
    	}
    	QueryAtomContainer q = new QueryAtomContainer();
    	q.setID(SULPHAMATE);
    	String atoms[] = {"S","N","O","O","O","H"}; 
    	SymbolQueryAtom[] a = new SymbolQueryAtom[6];
    	for (int i= 0; i<(atoms.length-1);i++) {
    		a[i] = new SymbolQueryAtom(new Atom(atoms[i]));
    		q.addAtom(a[i]);
    	}
    	q.addBond(new OrderQueryBond(a[0], a[1], CDKConstants.BONDORDER_SINGLE));//S-N
    	q.addBond(new OrderQueryBond(a[0], a[2], CDKConstants.BONDORDER_DOUBLE));//S=O
    	q.addBond(new OrderQueryBond(a[0], a[3], CDKConstants.BONDORDER_DOUBLE));//S=O
    	q.addBond(new OrderQueryBond(a[0], a[4], CDKConstants.BONDORDER_SINGLE));//S-O
    	//q.addBond(new OrderQueryBond(a[1], a[5], CDKConstants.BONDORDER_SINGLE));//N-H
    	if (me != null)
    		q.addBond(new OrderQueryBond(a[4], me, CDKConstants.BONDORDER_SINGLE));//O-Me
    	return q;

    }
    //R-C(=O)O-Me
    public static QueryAtomContainer saltOfCarboxylicAcid(String[] metals) {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(CARBOXYLIC_ACID_SALT);        
        SymbolSetQueryAtom m = new SymbolSetQueryAtom();
        for (int i=0; i < metals.length; i++)
            m.addSymbol(metals[i]);
        
        SymbolSetQueryAtom r = new SymbolSetQueryAtom();
        r.setSymbol("*");    r.addSymbol("C");      r.addSymbol("H");
        r.setProperty(DONTMARK,query.getID());
        
        SymbolQueryAtom c = new SymbolQueryAtom(new Atom("C"));
        SymbolQueryAtom o1 = new SymbolQueryAtom(new Atom("O"));
        SymbolQueryAtom o2 = new SymbolQueryAtom(new Atom("O"));
        query.addAtom(r); query.addAtom(c);
        query.addAtom(o1); query.addAtom(o2);
        query.addBond(new OrderQueryBond(r, c, CDKConstants.BONDORDER_SINGLE));//R-C
        query.addBond(new OrderQueryBond(c, o1, CDKConstants.BONDORDER_SINGLE));//C-O
        query.addBond(new OrderQueryBond(c, o2, CDKConstants.BONDORDER_DOUBLE));//C=O
        query.addBond(new OrderQueryBond(o1, m, CDKConstants.BONDORDER_SINGLE));//O-Me
        return query;
    }

    //C[O-] Me+
    public static QueryAtomContainer saltOfCarboxylicAcid1(String[] metals) {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(CARBOXYLIC_ACID_SALT);        
        SymbolSetQueryAtom m = new SymbolSetQueryAtom();
        for (int i=0; i < metals.length; i++)
            m.addSymbol(metals[i]);
        
        
        SymbolQueryAtom c = new SymbolQueryAtom(new Atom("C"));
        Atom a = new Atom("O");
        a.setFormalCharge(-1);
        SymbolAndChargeQueryAtom o1 = new SymbolAndChargeQueryAtom(a);
        query.addAtom(c);
        query.addAtom(o1);
        
        query.addBond(new OrderQueryBond(c, o1, CDKConstants.BONDORDER_SINGLE));//C-O
        query.addBond(new OrderQueryBond(o1, m, CDKConstants.BONDORDER_SINGLE));//O-Me
        return query;
    }
    //C[O-] Me+
    public static QueryAtomContainer saltOfCarboxylicAcid2(String[] metals) {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(CARBOXYLIC_ACID_SALT);        
        SymbolSetQueryAtom m = new SymbolSetQueryAtom();
        for (int i=0; i < metals.length; i++)
            m.addSymbol(metals[i]);
        
        
        SymbolQueryAtom c = new SymbolQueryAtom(new Atom("C"));
        Atom a = new Atom("C");
        a.setFormalCharge(-1);
        SymbolAndChargeQueryAtom o1 = new SymbolAndChargeQueryAtom(a);
        query.addAtom(c);
        query.addAtom(o1);
        
        query.addBond(new OrderQueryBond(c, o1, CDKConstants.BONDORDER_SINGLE));//C-O
        query.addBond(new OrderQueryBond(o1, m, CDKConstants.BONDORDER_SINGLE));//O-Me
        return query;
    }    
    
    public static QueryAtomContainer saltOfCarboxylicAcidBreakable(String[] metals) {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(CARBOXYLIC_ACID_SALT);        
        SymbolSetQueryAtom m = new SymbolSetQueryAtom();
        for (int i=0; i < metals.length; i++)
            m.addSymbol(metals[i]);
        m.setProperty(DONTMARK,query.getID());
        
        
        SymbolSetQueryAtom r = new SymbolSetQueryAtom();
        r.setSymbol("*");    r.addSymbol("C");      r.addSymbol("H");
        
        SymbolQueryAtom c = new SymbolQueryAtom(new Atom("C"));
        SymbolQueryAtom o1 = new SymbolQueryAtom(new Atom("O"));
                
        SymbolQueryAtom o2 = new SymbolQueryAtom(new Atom("O"));
        query.addAtom(r); query.addAtom(c);
        query.addAtom(o1); query.addAtom(o2);
        query.addBond(new OrderQueryBond(r, c, CDKConstants.BONDORDER_SINGLE));//R-C
        query.addBond(new OrderQueryBond(c, o1, CDKConstants.BONDORDER_SINGLE));//C-O
        query.addBond(new OrderQueryBond(c, o2, CDKConstants.BONDORDER_DOUBLE));//C=O
        query.addBond(new OrderQueryBond(o1, m, CDKConstants.BONDORDER_SINGLE));//O-Me
        return query;
    }
    
    //C-[O-] Me+
    public static QueryAtomContainer saltOfCarboxylicAcidBreakable1(String[] metals) {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(CARBOXYLIC_ACID_SALT);        
        SymbolSetQueryAtom m = new SymbolSetQueryAtom();
        for (int i=0; i < metals.length; i++)
            m.addSymbol(metals[i]);
        m.setProperty(DONTMARK,query.getID());
        
        
        SymbolQueryAtom c = new SymbolQueryAtom(new Atom("C"));
        Atom a = new Atom("O");
        a.setFormalCharge(-1);
        SymbolAndChargeQueryAtom o1 = new SymbolAndChargeQueryAtom(a);
                
        query.addAtom(c);
        query.addAtom(o1); 
        query.addBond(new OrderQueryBond(c, o1, CDKConstants.BONDORDER_SINGLE));//C-O
        query.addBond(new OrderQueryBond(o1, m, CDKConstants.BONDORDER_SINGLE));//O-Me
        return query;
    }
    public static QueryAtomContainer saltOfCarboxylicAcidBreakable2(String[] metals) {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(CARBOXYLIC_ACID_SALT);        
        SymbolSetQueryAtom m = new SymbolSetQueryAtom();
        for (int i=0; i < metals.length; i++)
            m.addSymbol(metals[i]);
        m.setProperty(DONTMARK,query.getID());
        
        
        SymbolQueryAtom c = new SymbolQueryAtom(new Atom("C"));
        Atom a = new Atom("C");
        a.setFormalCharge(-1);
        SymbolAndChargeQueryAtom o1 = new SymbolAndChargeQueryAtom(a);
                
        query.addAtom(c);
        query.addAtom(o1); 
        query.addBond(new OrderQueryBond(c, o1, CDKConstants.BONDORDER_SINGLE));//C-O
        query.addBond(new OrderQueryBond(o1, m, CDKConstants.BONDORDER_SINGLE));//O-Me
        return query;
    }        
    /*
    public static QueryAtomContainer sulphonate() {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(SULPHONATE);

        InverseSymbolSetQueryAtom r = new InverseSymbolSetQueryAtom();
        r.addSymbol("O");
        r.addSymbol("H");
        r.addSymbol("N");
        r.setProperty(DONTMARK,query.getID());
        SymbolQueryAtom s = new SymbolQueryAtom(new Atom("S"));
        Atom o = new Atom("O");
        o.setFormalCharge(-1);
//        SymbolAndChargeQueryAtom o1 = new SymbolAndChargeQueryAtom(o);
        //TODO verify which is the correct way of representing sulphonate, should nonionic be enabled?
        SymbolQueryAtom o1 = new SymbolQueryAtom(o);
        
        SymbolQueryAtom o2 = new SymbolQueryAtom(new Atom("O"));
        SymbolQueryAtom o3 = new SymbolQueryAtom(new Atom("O"));        
        query.addAtom(r); query.addAtom(s);
        query.addAtom(o1); query.addAtom(o2); query.addAtom(o3);
        query.addBond(new OrderQueryBond(r, s, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(s, o1, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(s, o2, CDKConstants.BONDORDER_DOUBLE));
        query.addBond(new OrderQueryBond(s, o3, CDKConstants.BONDORDER_DOUBLE));

        return query;
    }    
    */
    //RS(=O)(=O)O
    public static QueryAtomContainer sulphonate(String[] metals) {
    	return sulphonate(metals,true);
    }
    public static QueryAtomContainer sulphonate(String[] metals,boolean bonded) {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(SULPHONATE);
        SymbolSetQueryAtom m = null;
        if (metals != null) {
         	m = new SymbolSetQueryAtom();
	        for (int i=0; i < metals.length; i++)
	            m.addSymbol(metals[i]);
	        query.addAtom(m);
		}        
        //TODO moze li da ima izob sto atom razlicen ot C na towa miasto 
        InverseSymbolSetQueryAtom r = new InverseSymbolSetQueryAtom();
        r.addSymbol("O");
        r.addSymbol("H");
        r.addSymbol("N");
        
        
        r.setProperty(DONTMARK,query.getID());
        
        SymbolQueryAtom s = new SymbolQueryAtom(new Atom("S"));
        SymbolQueryAtom o1 = new SymbolQueryAtom(new Atom("O"));
        SymbolQueryAtom o2 = new SymbolQueryAtom(new Atom("O"));
        SymbolQueryAtom o3 = new SymbolQueryAtom(new Atom("O"));        
        query.addAtom(r); query.addAtom(s);
        query.addAtom(o1); query.addAtom(o2); query.addAtom(o3);
        query.addBond(new OrderQueryBond(r, s, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(s, o1, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(s, o2, CDKConstants.BONDORDER_DOUBLE));
        query.addBond(new OrderQueryBond(s, o3, CDKConstants.BONDORDER_DOUBLE));
        if ((m != null) && bonded)         
        	query.addBond(new OrderQueryBond(o1, m, CDKConstants.BONDORDER_SINGLE));
        return query;
    }
    //[setOfAtoms]-OS(O-subst)(=O)(=O)O
    public static QueryAtomContainer sulphate(String[] setOfAtoms) {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(SULPHATE);
        SymbolSetQueryAtom m = null;
        if (setOfAtoms != null) {
         	m = new SymbolSetQueryAtom();
	        for (int i=0; i < setOfAtoms.length; i++)
	            m.addSymbol(setOfAtoms[i]);
		}
        AnyAtom r = new AnyAtom();
        r.setProperty(DONTMARK,query.getID());
        
        SymbolQueryAtom s = new SymbolQueryAtom(new Atom("S"));
        SymbolQueryAtom o1 = new SymbolQueryAtom(new Atom("O"));
        SymbolQueryAtom o2 = new SymbolQueryAtom(new Atom("O"));
        SymbolQueryAtom o3 = new SymbolQueryAtom(new Atom("O"));
        SymbolQueryAtom o4 = new SymbolQueryAtom(new Atom("O"));          
         
        query.addAtom(s);query.addAtom(r);
        query.addAtom(o1); query.addAtom(o2); query.addAtom(o3);
        
        query.addBond(new OrderQueryBond(s, o1, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(s, o2, CDKConstants.BONDORDER_DOUBLE));
        query.addBond(new OrderQueryBond(s, o3, CDKConstants.BONDORDER_DOUBLE));
        query.addBond(new OrderQueryBond(s, o4, CDKConstants.BONDORDER_SINGLE));        
        if (m != null)         
        	query.addBond(new OrderQueryBond(o1, m, CDKConstants.BONDORDER_SINGLE));
        query.addBond(new OrderQueryBond(o4, r, CDKConstants.BONDORDER_SINGLE));
        return query;
    }
    
    //polyoxyethylene
    public static QueryAtomContainer polyoxyethylene(int n) {
        QueryAtomContainer query = new QueryAtomContainer();
        query.setID(POLYOXYETHYLENE);
        AnyAtom a1 = new AnyAtom();
        a1.setProperty(DONTMARK,query.getID());
        AnyAtom a2 = new AnyAtom();
        a2.setProperty(DONTMARK,query.getID());
        
        SymbolQueryAtom o;
        SymbolQueryAtom[] c = new SymbolQueryAtom[2];
        SymbolQueryAtom[] h = new SymbolQueryAtom[4];
        for (int i = 0; i < n; i++) {
            o = new SymbolQueryAtom(new Atom("O")); query.addAtom(o);
            if (i>0)
                query.addBond(new OrderQueryBond(o, c[1], CDKConstants.BONDORDER_SINGLE));
            else
                query.addBond(new OrderQueryBond(o, a1, CDKConstants.BONDORDER_SINGLE));            	
            for (int j=0; j < 2; j++) { 
                c[j] = new SymbolQueryAtom(new Atom("C")); query.addAtom(c[j]);
                for (int j1=0; j1 < 2; j1++) { 
                    h[j*2+j1] = new SymbolQueryAtom(new Atom("H")); 
                    query.addAtom(h[j*2+j1]);
                    query.addBond(new OrderQueryBond(c[j], h[j*2+j1], CDKConstants.BONDORDER_SINGLE));
                }
            }    
            query.addBond(new OrderQueryBond(o, c[0], CDKConstants.BONDORDER_SINGLE));
            query.addBond(new OrderQueryBond(c[0], c[1], CDKConstants.BONDORDER_SINGLE));
            if (i==(n-1))
                query.addBond(new OrderQueryBond(a2, c[1], CDKConstants.BONDORDER_SINGLE));
        }
        return query;
        
    }   
    public static IAtomContainer cloneDiscardRingAtomAndBonds(
    		IAtomContainer ac, IRing ring ) {
        IAtomContainer result = new org.openscience.cdk.AtomContainer();
        Hashtable table = new Hashtable();
        for (int i =0; i < ac.getAtomCount(); i++) {
        	IAtom a = ac.getAtomAt(i);
            if (ring.contains(a)) {
            	//if (!a.getSymbol().equals("C")) continue;
            	continue;
            }
            //Atom newAtom = (Atom) a.clone();
            table.put(a,a);
            result.addAtom(a);
        }
        IAtom[] atoms;
        for (int i =0; i < ac.getBondCount(); i++) {
            IBond b = ac.getBondAt(i);
            atoms = b.getAtoms();
            
            Atom a1 = (Atom) table.get(atoms[0]);
            if (a1 == null) continue;
            Atom a2 = (Atom) table.get(atoms[1]);
            if (a2 == null) continue;
            //Bond newBond = new org.openscience.cdk.Bond(a1, a2, b.getOrder());
            //newBond.setFlag(CDKConstants.ISAROMATIC,b.getFlag(CDKConstants.ISAROMATIC));
            //if (ring.contains(a1) && ring.contains(a2)) continue;
            result.addBond(b);
        }
        return result;
    }    
    // discards ring atom and bonds
    /*
    public static AtomContainer cloneDiscardRingAtomAndBonds(AtomContainer ac, Ring ring) {
        AtomContainer result = new AtomContainer();
        Hashtable table = new Hashtable();
        for (int i =0; i < ac.getAtomCount(); i++) {
            Atom a = ac.getAtomAt(i);
            if (ring.contains(a)) continue;
            Atom newAtom = (Atom) a.clone();
            table.put(a,newAtom);
            result.addAtom(newAtom);
        }
        Atom[] atoms;
        for (int i =0; i < ac.getBondCount(); i++) {
            Bond b = ac.getBondAt(i);
            atoms = b.getAtoms();
            Atom a1 = (Atom) table.get(atoms[0]);
            if (a1 == null) continue;
            Atom a2 = (Atom) table.get(atoms[1]);
            if (a2 == null) continue;
            Bond newBond = new org.openscience.cdk.Bond(a1, a2, b.getOrder());
            newBond.setFlag(CDKConstants.ISAROMATIC,b.getFlag(CDKConstants.ISAROMATIC));
            result.addBond(newBond);
        }
        return result;
    }
    */
    /*
    public static AtomContainer cloneDiscardRingBo nds(AtomContainer ac, Ring ring) {
        AtomContainer result = new AtomContainer();
        Hashtable table = new Hashtable();
        for (int i =0; i < ac.getAtomCount(); i++) {
            Atom a = ac.getAtomAt(i);
            if (ring.contains(a)) continue;
            Atom newAtom = (Atom) a.clone();
            table.put(a,newAtom);
            result.addAtom(newAtom);
        }
        Atom[] atoms;
        for (int i =0; i < ac.getBondCount(); i++) {
            Bond b = ac.getBondAt(i);
            atoms = b.getAtoms();
            Atom a1 = (Atom) table.get(atoms[0]);
            if (a1 == null) continue;
            Atom a2 = (Atom) table.get(atoms[1]);
            if (a2 == null) continue;
            Bond newBond = new Bond(a1, a2, b.getOrder());
            newBond.setFlag(CDKConstants.ISAROMATIC,b.getFlag(CDKConstants.ISAROMATIC));
            result.addBond(newBond);
        }
        return result;
    }
    */
    /**
     * 
     * @param mol - The molecule to be searched  {@link AtomContainer}
     * @param q - The query {@link AtomContainer}
     * @param isPreprocessed - whether the molecule was preprocessed by {@link #preProcess(IAtomContainer)}
     * @return List {@link List}
     */
    public static List getBondMap(IAtomContainer mol, IAtomContainer q,boolean isPreprocessed) {
    		logger.debug("getBondMap\t",q.getID());
        	List list = null;
			if (!isPreprocessed && needsPreprocessing(q)) {
				preProcess(mol);
				isPreprocessed = true;
			}        	
        	try {
	            list = UniversalIsomorphismTester.getSubgraphMaps(mol,q);
		    	markMaps(mol,q,list);
	    		
        	} catch (Exception x) {
        	    logger.error(x);
        	    list = null;
        	}
        	
    		
    		return list;
    }
    protected static String getKeyFromMap(RMap rmap) {
		return rmap.getId1() + "-" + rmap.getId2();	
    }
    protected static boolean isOverlapped(
			IAtomContainer mol, 
			IAtomContainer q, List first) {
        String id = q.getID();
        boolean overlaped = false;
		for (int i = 0; i < first.size(); i++) {
			RMap rmap = (RMap)first.get(i);
			IBond b1 = ((Bond) mol.getBondAt(rmap.getId1()));
			IBond b2 = ((Bond) q.getBondAt(rmap.getId2()));
			
			Object o = b2.getProperty(DONTMARK);
			if ((o != null) && (o.equals(id))) continue;
				
			o = b1.getProperty(ALLOCATED);
			if ((o != null) && (!o.equals(id))) {
				logger.debug("Possible overlap of group\t",id,"\tand\t",o.toString(),"\t");
				overlaped = true;
				break;
			}
		}
		return overlaped;

    }
    
    public static List getUniqueBondMap(IAtomContainer mol, IAtomContainer q, boolean isPreprocessed) {
    	if (q.getAtomCount() > mol.getAtomCount()) {
    		logger.debug("A query with more atoms than a molecule!");
    		return null;
    	}
    	List list = null;
    	//try {
    		//this is bonds map
			if (!isPreprocessed && needsPreprocessing(q)) {
				preProcess(mol);
				isPreprocessed = true;
			}    	
			try {
				list = UniversalIsomorphismTester.getSubgraphMaps(mol,q);
			} catch (CDKException x) {
				list = null;
				logger.error(x);
			}
            if ((list == null) || (list.size()==0))
            	logger.debug("getUniqueBondMap\t",q.getID(),"\t",AbstractRule.MSG_NO);
            else
            	logger.debug("getUniqueBondMap\t",q.getID(),"\t",AbstractRule.MSG_YES);
            if (list.size() < 2) return list;
            //else delete redundant solutions
            TreeMap all = new TreeMap(); //to contain all redundant solutions
            TreeMap aMap = new TreeMap(); //current analyzed list of Rmap in string form

            //this is to get rid of symmetric and overlaping solutions (if any)
            boolean[] keep = new boolean[list.size()];
            int[] mappedBonds = new int[mol.getBondCount()];
            for (int j=0; j < mappedBonds.length; j++) mappedBonds[j] = -1;
            
            //TODO have to take care if ArrayList of RMap is returned instead of  ArrayList of ArrayList of RMap
            logger.debug(FunctionalGroups.mapToString(mol));
            
    		for (int j = list.size()-1; j >=0 ; j--) 
    			if (isOverlapped(mol, q,(List)list.get(j))) {
    				logger.debug("Remove overlaped group ",q.getID(),"");
    				list.remove(j);
    			}            
            
    		for (int j = 0; j < list.size(); j++) {
    			List thisMap = (List)list.get(j);
    			aMap.clear();
    			for (int i = 0; i < thisMap.size(); i++) {
    				RMap rmap = (RMap)thisMap.get(i);
    				
    				IAtom[] a2 = ((IBond) q.getBondAt(rmap.getId2())).getAtoms();
    				
    				if (mappedBonds[rmap.getId1()] > 0) {
    					//this bond is already mapped (i.e. overlapped patterns)
    					//clear mark for bonds only in the current map
    					for (int k=0; k < mappedBonds.length; k++)
    						if (mappedBonds[k]==0) mappedBonds[k] =-1;
    					aMap.clear();
    					break;
    				} else {
    					//mark  bonds in the current map

        				if (a2[0].getSymbol().equals("R")) continue;
        				if (a2[1].getSymbol().equals("R")) continue;

        				/*
        				Object o = a2[0].getProperty(DONTMARK);
        				if (o != null) continue;
        				o = a2[1].getProperty(DONTMARK);
        				if (o != null) continue;
        				*/
        				
        				mappedBonds[rmap.getId1()] = 0;        		
        				//if (a2[0].getSymbol().equals("H")) continue;
        				//if (a2[1].getSymbol().equals("H")) continue;
    							
    					aMap.put(getKeyFromMap(rmap),rmap);
    				}
    			}
    			for (int k=0; k < mappedBonds.length; k++)
					if (mappedBonds[k]==0) mappedBonds[k] =1;    			
    			if (aMap.size() == 0) {
    				keep[j] = false; continue;
    			} else {
	    			String bondMap = aMap.keySet().toString();
	    			
	    			//logger.debug(bondMap);
	    			if 	(all.containsKey(bondMap))
	    				keep[j] = false;
	    			else {
	    				keep[j] = true;
	    				all.put(bondMap,thisMap);
	    			}
    			}
    		}
    		for (int j = (list.size()-1); j >=0 ; j--) 
    			if (!keep[j]) list.remove(j);
    		
    		/*
    	} catch (CDKException x) {
    	    x.printStackTrace();
    	    list = null;
    	}
    	*/
    	//markMaps(mol,q,list);
		return list;

    }
    public static int mark(IAtomContainer mol,ArrayList elements) {
    	int hydroCarbonCount = 0;
    	for (int i =0; i < mol.getAtomCount(); i++) {
    		IAtom a = mol.getAtomAt(i);
    		if (elements.contains(a.getSymbol())) 
    			a.setProperty(a.getSymbol(),new Integer(i));
    	}
    	return hydroCarbonCount;
    }
    
    public static int markCHn(IAtomContainer mol) {
    	IAtom[] neighbors = null;
    	int hydroCarbonCount = 0;
    	for (int i =0; i < mol.getAtomCount(); i++) {
    		IAtom a = mol.getAtomAt(i);
    		if (!a.getSymbol().equals("C")) continue;
    		neighbors = mol.getConnectedAtoms(a);
    		
    		int hydrogens = 0;
    		int carbons = 0;
    		for (int j=0; j < neighbors.length; j++)
    			if (neighbors[j].getSymbol().equals("H")) hydrogens++;
    			else if (neighbors[j].getSymbol().equals("C")) carbons++;
    			
    		String key = "";
    		switch (hydrogens) {
    		case 0: {
    			if (carbons==neighbors.length)
    			key = C; break;
    		}
    		case 1: {
    			key = CH; break;
    		}
    		case 2: {
    			key = CH2;break;
    		}
    		case 3: {
    			key = CH3;break;
    			
    		}
    		default : {
    		}
    		}
   			hydroCarbonCount++;
   			a.setProperty(key,new Integer(hydroCarbonCount));
    	}
    	return hydroCarbonCount;
    }
    public static void markOneMap(
			IAtomContainer mol, 
			IAtomContainer q, List first, int j) {
        String id = q.getID();
		for (int i = 0; i < first.size(); i++) {
			RMap rmap = (RMap)first.get(i);
			IBond b2 = q.getBondAt(rmap.getId2());
			IAtom[] a2 = b2.getAtoms();
			
			//will not mark bonds that does not belong to the group but to the connected fragments
			boolean markIt = true;
			for (int k=0; k<a2.length;k++) {
				if (a2[k] instanceof AnyAtom) {
					markIt = false; break;
				}
				Object o =a2[k].getProperty(DONTMARK);
				if (o != null) {
					markIt = false;break;
				}
			}
			if (!markIt) continue; //will not mark this bond
			
			Integer m = new Integer(j+1);
			IBond b1 = mol.getBondAt(rmap.getId1());
			b1.setProperty(id,m);
			b1.setProperty(ALLOCATED,id);			
			IAtom[] a1 = b1.getAtoms();	    				
			
			for (int k=0; k < a1.length; k++) {
				a1[k].setProperty(id,m);
				a1[k].setProperty(ALLOCATED,id);
			}	
		}

    }
    
    public static void markMaps(
    			IAtomContainer mol, 
    			IAtomContainer q, List list) {
    		if (list == null) return;

			if ((list.size()>0) && (list.get(0) instanceof RMap)) {
				markOneMap(mol,q,list,1);
			} else
	    		for (int j = 0; j < list.size(); j++) {
	    			List first = (List)list.get(j);
	    			markOneMap(mol,q,first,j);
	    		}	
    }
    
    public static void mapHydrocarbon(IMolecule mol) {
        for (int i=0; i < mol.getAtomCount(); i++) {
            
        }
    }
    public static StringBuffer mapToString(IAtomContainer mol,String id) {
        StringBuffer b = new StringBuffer();
        b.append("Atoms\t");
        for (int i =0; i < mol.getAtomCount(); i++) {
            Object o = mol.getAtomAt(i).getProperty(id);
            b.append((i+1));
            b.append('\t');
            b.append(mol.getAtomAt(i).getSymbol());
            b.append('\t');
            b.append(id);
            b.append('\t');
            b.append(o);
            b.append('\t');
        }
        b.append("Bonds\t");
        for (int i =0; i < mol.getBondCount(); i++) {
            Object o = mol.getBondAt(i).getProperty(id);
            b.append((i+1));
            b.append('\t');
            b.append(mol.getBondAt(i).getAtomAt(0).getSymbol());
            b.append('-');
            b.append(mol.getBondAt(i).getAtomAt(1).getSymbol());
            b.append('\t');            
            b.append(id);
            b.append('\t');
            b.append(o);
            b.append('\t');
        }            
        return b;
    }
    
    public static StringBuffer mapToString(IAtomContainer mol,
    		Collection id) {
        StringBuffer b = new StringBuffer();
        boolean[] f = null;
        IAtom a = null;
        for (int i =0; i < mol.getAtomCount(); i++) {
        	a = mol.getAtomAt(i);
            b.append((i+1));
            b.append('\t');
            b.append(a.getSymbol());
            f = a.getFlags();
            for (int j=0; j < f.length; j++) if (f[j])
            	switch (j) {
            	case (CDKConstants.ISALIPHATIC): { b.append("\taliphatic\t"); break;}
            	case (CDKConstants.ISAROMATIC): { b.append("\taromatic\t"); break;}
            	case (CDKConstants.ISINRING): { b.append("\tin ring\t"); break;}
            	}
                        
            Iterator iterator = id.iterator();
            while(iterator.hasNext()) {
            	Object anID = iterator.next();
	            Object o = a.getProperty(anID);
	            if (o!=null) {
		            b.append('\t');
		            b.append(anID);
		            b.append('\t');
		            b.append(o);
	            }
            }
	        b.append('\t');            
        }    
        return b;
    }
    
    public static StringBuffer mapToString(IAtomContainer mol) {
        StringBuffer b = new StringBuffer();
        boolean[] f = null;
        for (int i =0; i < mol.getAtomCount(); i++) {
        	IAtom a = mol.getAtomAt(i);
            b.append((i+1));
            b.append(')');
            b.append(a.getSymbol());
            if (a.getFormalCharge() != 0) b.append(a.getFormalCharge()); 
            b.append(' ');

            f = a.getFlags();
            for (int j=0; j < f.length; j++) if (f[j])
            	switch (j) {
            	case (CDKConstants.ISALIPHATIC): { b.append("[aliphatic] "); break;}
            	case (CDKConstants.ISAROMATIC): { b.append("[aromatic] "); break;}
            	case (CDKConstants.ISINRING): { b.append("[in ring] "); break;}
            	}
            
           	Hashtable h = a.getProperties();
            for (Enumeration e = h.keys();e.hasMoreElements();) {
            	Object key = e.nextElement();
	            Object o = a.getProperty(key);
	            if (o!=null) {
		            b.append('[');
		            b.append(key);
		            b.append("] ");
		            b.append(o);
	            }
            }

	        b.append(' ');

        }
        b.append("\nBonds\t");
        for (int i =0; i < mol.getBondCount(); i++) {
            b.append((i+1));
            b.append('\t');
            b.append(mol.getBondAt(i).getAtomAt(0).getSymbol());
            b.append('-');
            b.append(mol.getBondAt(i).getAtomAt(1).getSymbol());
            b.append('\t');
           	Hashtable h = mol.getBondAt(i).getProperties();
            for (Enumeration e = h.keys();e.hasMoreElements();) {
            	Object key = e.nextElement();
	            Object o = mol.getBondAt(i).getProperty(key);
	            if (o!=null) {
		            b.append('[');
		            b.append(key);
		            b.append("] ");
		            b.append(o);
	            }
            }            
            b.append('\t');
        }        
        return b;
    }    
            
    /*
    public static boolean isSubstructure(AtomContainer mol, AtomContainer query ) throws CDKException {
		logger.debug("Checking if has substructure \t",query.getID());
		if (UniversalIsomorphismTester.isSubgraph(mol,query)) {
			logger.debug("Substructure FOUND \t",query.getID());
			return true;
		} else {
			logger.debug("Substructure NOT found \t",query.getID());
			return false;		 
		}    	
    }
    */
    /**
     * @param mol
     * @param query ArrayList of AtomContainer  - - the list of substructures to be searched for
     * @return true if any of the fragments are a substructure of mol
     * @throws CDKException
     */
    public static boolean hasAnySubstructure(IAtomContainer mol, ArrayList query ) throws CDKException {
	    if ((query == null) || (query.size() == 0)) return false;
	    else
	    	for (int i=0; i < query.size(); i++) 
	    		if (FunctionalGroups.hasGroup(mol, (IAtomContainer) query.get(i))) return true;
		return false;	    
    }
    /**
     * @param mol
     * @param query ArrayList of AtomContainer  - - the list of substructures to be searched for
     * @return true if all of the fragments are a substructure of mol
     * @throws CDKException
     */
    public static boolean hasAllSubstructure(IAtomContainer mol, ArrayList query, boolean isPreprocessed) throws CDKException {
	    if ((query == null) || (query.size() == 0)) return false;
	    else
	    	for (int i=0; i < query.size(); i++) {
	    		IAtomContainer q = (IAtomContainer) query.get(i);
	    		if (!isPreprocessed && needsPreprocessing(q)) {
	    			preProcess(mol);
	    			isPreprocessed = true;
	    		}
	    		if (!FunctionalGroups.hasGroup(mol, q,false)) return false;
	    	}
		return true;	    
    }
    protected static boolean needsPreprocessing(IAtomContainer query) {
    	String id = query.getID();
    	return  (id != null) && 
			(
			(id.equals(ALCOHOL)) || 
			(id.equals(ETHER)) ||
			(id.equals(SECONDARY_AMINE_ALIPHATIC)) ||
			(id.equals(SECONDARY_AMINE)) ||
			(id.equals(LACTONE))
			)
			;
    }
    public static void preProcess(IAtomContainer mol) {
    	markCHn(mol);
    }
    public static boolean markUniqueBondMap(IMolecule mol, ArrayList query, boolean isPreprocessed) {
	    if ((query == null) || (query.size() == 0)) return false;
	    else
	    	for (int i=0; i < query.size(); i++) {
	    		QueryAtomContainer q = (QueryAtomContainer) query.get(i);
	    		if (!isPreprocessed && needsPreprocessing(q)) {
	    			preProcess(mol);
	    			isPreprocessed = true;
	    		}	    		
	    		List list = FunctionalGroups.getUniqueBondMap(mol, q,isPreprocessed);
	    		FunctionalGroups.markMaps(mol, q, list);
	    	}
		return true;	    
    }
    public static boolean hasOnlyTheseGroups(IAtomContainer  mol,
    			QueryAtomContainers query,Collection ids,boolean isPreprocessed) {
    	final String MSG = "Has only these groups\t";
	    if ((query == null) || (query.size() == 0)) return false;
	    else {
	    	logger.debug("Checking if it has only these groups\t",ids);
	    	List list = null;
	    	int found = 0;
	  	    for (int i=0; i < query.size(); i++) {
	    		AtomContainer q = (AtomContainer) query.get(i);
	    		list = FunctionalGroups.getUniqueBondMap(mol, q,isPreprocessed);
	    		if ((list == null) || (list.size() == 0)) continue;
	    		FunctionalGroups.markMaps(mol, q, list);
	    		logger.debug(mapToString(mol));
	    		if (hasMarkedOnlyTheseGroups(mol,ids)) {
	    			logger.info(MSG,ids,"\tYES");
	  				return true;
	    		}
	    		else found ++;
	    	}
	  	    if (logger.isDebugEnabled()) {
	  	    	logger.debug(mapToString(mol,ids));
	  	    }
  			if (hasMarkedOnlyTheseGroups(mol,ids)) {
  				logger.info(MSG,ids,"\tYES");
  				return true;
  			} else {
  				logger.info("Has only these groups\t",ids,"\tNO");
  				return false;
  			}
	    }
    }
    /**
     * Has to be preprocessed with getBondMap or getUniqueBondMap and markMaps
     * 
     * @param mol
     * @param id Collection of functional group identifiers as used by the procedures listed above
     * @return true if mol contains only specified groups (i.e. there are no unmarked atoms)
     */
    
    public static boolean hasMarkedOnlyTheseGroups(IAtomContainer mol,Collection id) {
    	IAtom a = null;
        
        for (int i =0; i < mol.getAtomCount(); i++) {
        	a = mol.getAtomAt(i);
        	if (a.getSymbol().equals("H")) continue;
        	if (a.getSymbol().equals("C")) continue;
            Iterator iterator = id.iterator();
            boolean ok = false; 
            while(iterator.hasNext()) {
            	Object anID = iterator.next();
	            Object o = a.getProperty(anID);
	            if (o!=null) {
	            	//this atom has at least one mark, no need to check for more
	            	ok = true; break;
	            }
            }
            
            if (!ok)  {
            	logger.debug("Atom "+ a.getSymbol() + " belongs to a forbidden group");
            	logger.debug(mapToString(mol,id));
            	return false; //at least one atom is unmarked
            }
        }
        logger.debug("No forbidden groups found");
        return true;
    }
    public static boolean hasGroupMarked(IAtomContainer mol,String id) {
        for (int i =0; i < mol.getAtomCount(); i++)
        	if (mol.getAtomAt(i).getProperty(id) != null) return true;
        return false;
    }    
    public static boolean hasMarkedOnlyTheseGroups(IAtomContainer mol,QueryAtomContainers query) {
    	IAtom a = null;
        
        for (int i =0; i < mol.getAtomCount(); i++) {
        	a = mol.getAtomAt(i);
        	if (a.getSymbol().equals("H")) continue;
            Iterator iterator = query.iterator();
            boolean ok = false; 
            while(iterator.hasNext()) {
            	QueryAtomContainer q = (QueryAtomContainer) iterator.next();
	            Object o = a.getProperty(q.getID());
	            if (o!=null) {
	            	//this atom has at least one mark, no need to check for more
	            	ok = true; break;
	            }
            }
            
            if (!ok)  {
            	
            	logger.debug("Atom "+ a.toString() + " belongs to a forbidden group");
            	
            	return false; //at least one atom is unmarked
            }
        }
        logger.debug("No forbidden groups found");
        return true;
    }
    public static IAtomContainer createAtomContainer(String smiles, String id) {
    	return createAtomContainer(smiles,false,id);
    }    
    public static IAtomContainer createAtomContainer(String smiles) {
    	return createAtomContainer(smiles,false,smiles);
    }
    public static IAtomContainer createAtomContainer(String smiles,boolean addHydrogens) {
    	return createAtomContainer(smiles,addHydrogens,smiles);
    }
    
    public static IAtomContainer createAtomContainer(String smiles,boolean addHydrogens,String id) {
        try {
        	//logger.debug("Creating molecule from SMILES\t",smiles);
        	SmilesParser sp = new SmilesParser();
        	
            IAtomContainer mol = sp.parseSmiles(smiles);
            if ((mol != null) && (mol.getAtomCount()==0)) return null;
            if (addHydrogens) 
            try {
            	//logger.debug("Adding explicit hydrogens");
                h.addExplicitHydrogensToSatisfyValency((IMolecule)mol);
            } catch (InvalidSmilesException x ) {
                logger.error(x);
                return null;
            } catch (CDKException x) {
            	logger.error(x);
                return null;
            } catch (ClassNotFoundException x) {
            	logger.error(x);
                return null;
            } catch (IOException x) {
            	logger.error(x);
                return null;
            }
                    
            mol.setID(id);
			sp = null;
			return mol;
        } catch (InvalidSmilesException x ) {
            logger.error(x);
            return null;
        }
    }         
    public static QueryAtomContainer createQuery(String smiles) {
    	return createQuery(smiles,smiles);
    }
    public static QueryAtomContainer createQuery(String smiles,String id) {
    	IAtomContainer mol = createAtomContainer(smiles);
    	if (mol !=null) {
    		QueryAtomContainer q = QueryAtomContainerCreator.createBasicQueryContainer(mol);
    		q.setID(id);
    		return q;
    	}	
    	else return null;
    }       
    public static QueryAtomContainer vicinalDiKetone() {
    	//TODO "[*]C(=O)C(=O)[*]"
    	  	
    	  QueryAtomContainer q = new QueryAtomContainer();
    	  q.setID("vicinal diketone");
    	  SymbolQueryAtom a1 = new SymbolQueryAtom(new Atom("O"));
    	  q.addAtom(a1);
    	  SymbolQueryAtom a2 = new SymbolQueryAtom(new Atom("C"));
    	  q.addAtom(a2);
    	  InverseSymbolSetQueryAtom a3 = new InverseSymbolSetQueryAtom();
    	  a3.addSymbol("O"); a3.addSymbol("S"); q.addAtom(a3);
    	  SymbolQueryAtom a4 = new SymbolQueryAtom(new Atom("C"));
    	  q.addAtom(a4);
    	  SymbolQueryAtom a5 = new SymbolQueryAtom(new Atom("O"));
    	  q.addAtom(a5);
    	  InverseSymbolSetQueryAtom a6 = new InverseSymbolSetQueryAtom();
    	  a6.addSymbol("O"); a6.addSymbol("S");  q.addAtom(a6);
    	  OrderQueryBond b1 = new OrderQueryBond(a2, a1, 2.0);
    	  q.addBond(b1);
    	  OrderQueryBond b2 = new OrderQueryBond(a3, a2, 1.0);
    	  q.addBond(b2);
    	  OrderQueryBond b3 = new OrderQueryBond(a4, a2, 1.0);
    	  q.addBond(b3);
    	  OrderQueryBond b4 = new OrderQueryBond(a5, a4, 2.0);
    	  q.addBond(b4);
    	  OrderQueryBond b5 = new OrderQueryBond(a6, a4, 1.0);
    	  q.addBond(b5);    	
    	  return q;
    }
    public static QueryAtomContainer ketoneAttachedToTerminalVinyl() {
    	  QueryAtomContainer mol = new   QueryAtomContainer();
    	  mol.setID("Ketone attached to terminal vinyl");
    	  SymbolQueryAtom c1 = new SymbolQueryAtom(new Atom("C"));
    	  mol.addAtom(c1);
    	  InverseSymbolSetQueryAtom r = new InverseSymbolSetQueryAtom();
    	  r.addSymbol("O");  r.addSymbol("S");  mol.addAtom(r);
    	  SymbolQueryAtom c2 = new SymbolQueryAtom(new Atom("C"));
    	  mol.addAtom(c2);
    	  SymbolQueryAtom o = new SymbolQueryAtom(new Atom("O"));
    	  mol.addAtom(o);
    	  SymbolQueryAtom c3 = new SymbolQueryAtom(new Atom("C"));
    	  mol.addAtom(c3);
    	  SymbolQueryAtom h1 = new SymbolQueryAtom(new Atom("H"));
    	  mol.addAtom(h1);
    	  SymbolQueryAtom h2 = new SymbolQueryAtom(new Atom("H"));
    	  mol.addAtom(h2);
    	  SymbolQueryAtom h3 = new SymbolQueryAtom(new Atom("H"));
    	  mol.addAtom(h3);
    	  OrderQueryBond b1 = new OrderQueryBond(r, c1, 1.0);
    	  mol.addBond(b1);
    	  OrderQueryBond b2 = new OrderQueryBond(c1, c2, 1.0);
    	  mol.addBond(b2);
    	  OrderQueryBond b3 = new OrderQueryBond(c1,o, 2.0);
    	  mol.addBond(b3);
    	  OrderQueryBond b4 = new OrderQueryBond(c2, c3, 2.0);
    	  mol.addBond(b4);
    	  OrderQueryBond b5 = new OrderQueryBond(c3, h1, 1.0);
    	  mol.addBond(b5);
    	  OrderQueryBond b6 = new OrderQueryBond(c3, h2, 1.0);
    	  mol.addBond(b6);
    	  OrderQueryBond b7 = new OrderQueryBond(c2,h3, 1.0);
    	  mol.addBond(b7);
    	  return mol;
    }
    public static QueryAtomContainer ketalAttachedToTerminalVinyl() {
    	//TODO "[*]C(O)(O)C=C"
    	  QueryAtomContainer mol = new   QueryAtomContainer();
    	  mol.setID("ketal attached to terminal vinyl");
    	  InverseSymbolSetQueryAtom r1 = new  InverseSymbolSetQueryAtom();
    	  r1.addSymbol("O");   r1.addSymbol("H");   mol.addAtom(r1);
    	  InverseSymbolSetQueryAtom r2 = new  InverseSymbolSetQueryAtom();
    	  r2.addSymbol("O");   r2.addSymbol("H");   mol.addAtom(r2);
    	  InverseSymbolSetQueryAtom r3 = new  InverseSymbolSetQueryAtom();
    	  r3.addSymbol("O");   r3.addSymbol("H");   mol.addAtom(r3);
    	  
    	    SymbolQueryAtom a1 = new SymbolQueryAtom(new Atom("C"));
    	    mol.addAtom(a1);
    	    SymbolQueryAtom a3 = new SymbolQueryAtom(new Atom("C"));
    	    mol.addAtom(a3);
    	    SymbolQueryAtom a4 = new SymbolQueryAtom(new Atom("O"));
    	    mol.addAtom(a4);
    	    SymbolQueryAtom a5 = new SymbolQueryAtom(new Atom("O"));
    	    mol.addAtom(a5);
    	    SymbolQueryAtom a6 = new SymbolQueryAtom(new Atom("C"));
    	    mol.addAtom(a6);
    	    SymbolQueryAtom a9 = new SymbolQueryAtom(new Atom("H"));
    	    mol.addAtom(a9);
    	    SymbolQueryAtom a10 = new SymbolQueryAtom(new Atom("H"));
    	    mol.addAtom(a10);
    	    OrderQueryBond b1 = new OrderQueryBond(r1, a1, 1.0);
    	    mol.addBond(b1);
    	    OrderQueryBond b2 = new OrderQueryBond(a1, a3, 1.0);
    	    mol.addBond(b2);
    	    OrderQueryBond b3 = new OrderQueryBond(a1, a4, 1.0);
    	    mol.addBond(b3);
    	    OrderQueryBond b4 = new OrderQueryBond(a1, a5, 1.0);
    	    mol.addBond(b4);
    	    OrderQueryBond b5 = new OrderQueryBond(a3, a6, 2.0);
    	    mol.addBond(b5);
    	    OrderQueryBond b6 = new OrderQueryBond(a4, r2, 1.0);
    	    mol.addBond(b6);
    	    OrderQueryBond b7 = new OrderQueryBond(a5, r3, 1.0);
    	    mol.addBond(b7);
    	    OrderQueryBond b8 = new OrderQueryBond(a6, a9, 1.0);
    	    mol.addBond(b8);
    	    OrderQueryBond b9 = new OrderQueryBond(a6, a10, 1.0);
    	    mol.addBond(b9);
    	    return mol;
    }  
    public static QueryAtomContainer alcoholSecondaryAttachedToTerminalVinyl() {
    	//TODO "[*]C(O)C=C"
    	//return createQuery("CC(O)C=C",);
    	//return createQuery("[H]OC([H])(C)C([H])=C([H])[H]","secondary alcohol attached to terminal vinyl group");
    	  IMolecule mol = new org.openscience.cdk.Molecule();
    	  Atom a1 = new Atom("H");
    	  mol.addAtom(a1);
    	  Atom a2 = new Atom("O");
    	  mol.addAtom(a2);
    	  Atom a3 = new Atom("C");
    	  mol.addAtom(a3);
    	  Atom a4 = new Atom("H");
    	  mol.addAtom(a4);
    	  Atom a5 = new Atom("C");
    	  mol.addAtom(a5);
    	  Atom a6 = new Atom("C");
    	  mol.addAtom(a6);
    	  Atom a7 = new Atom("H");
    	  mol.addAtom(a7);
    	  Atom a8 = new Atom("C");
    	  mol.addAtom(a8);
    	  Atom a9 = new Atom("H");
    	  mol.addAtom(a9);
    	  Atom a10 = new Atom("H");
    	  mol.addAtom(a10);
    	  Bond b1 = new org.openscience.cdk.Bond(a2, a1, 1.0);
    	  mol.addBond(b1);
    	  Bond b2 = new org.openscience.cdk.Bond(a3, a2, 1.0);
    	  mol.addBond(b2);
    	  Bond b3 = new org.openscience.cdk.Bond(a4, a3, 1.0);
    	  mol.addBond(b3);
    	  Bond b4 = new org.openscience.cdk.Bond(a5, a3, 1.0);
    	  mol.addBond(b4);
    	  Bond b5 = new org.openscience.cdk.Bond(a6, a3, 1.0);
    	  mol.addBond(b5);
    	  Bond b6 = new org.openscience.cdk.Bond(a7, a6, 1.0);
    	  mol.addBond(b6);
    	  Bond b7 = new org.openscience.cdk.Bond(a8, a6, 2.0);
    	  mol.addBond(b7);
    	  Bond b8 = new org.openscience.cdk.Bond(a9, a8, 1.0);
    	  mol.addBond(b8);
    	  Bond b9 = new org.openscience.cdk.Bond(a10, a8, 1.0);
    	  mol.addBond(b9);
    	  QueryAtomContainer q = QueryAtomContainerCreator.createBasicQueryContainer(mol);
  		  q.setID("secondary alcohol attached to terminal vinyl group");
  		  return q;    	  
    	
    	
    }     
    public static QueryAtomContainer esterOfalcoholSecondaryAttachedToTerminalVinyl() {
    	//TODO ""CC(C(=O)O[*])C=C""
    	//return createQuery("CC(C(=O)OC)C=C","ester of secondary alcohol attached to terminal vinyl group");
    	  IMolecule mol = new org.openscience.cdk.Molecule();
    	  Atom a1 = new Atom("C");
    	  mol.addAtom(a1);
    	  Atom a2 = new Atom("C");
    	  mol.addAtom(a2);
    	  Atom a3 = new Atom("H");
    	  mol.addAtom(a3);
    	  Atom a4 = new Atom("H");
    	  mol.addAtom(a4);
    	  Atom a5 = new Atom("H");
    	  mol.addAtom(a5);
    	  Atom a6 = new Atom("C");
    	  mol.addAtom(a6);
    	  Atom a7 = new Atom("C");
    	  mol.addAtom(a7);
    	  Atom a8 = new Atom("C");
    	  mol.addAtom(a8);
    	  Atom a9 = new Atom("O");
    	  mol.addAtom(a9);
    	  Atom a10 = new Atom("O");
    	  mol.addAtom(a10);
    	  Atom a11 = new Atom("C");
    	  mol.addAtom(a11);
    	  Atom a12 = new Atom("H");
    	  mol.addAtom(a12);
    	  Bond b1 = new org.openscience.cdk.Bond(a1, a2, 2.0);
    	  mol.addBond(b1);
    	  Bond b2 = new org.openscience.cdk.Bond(a1, a3, 1.0);
    	  mol.addBond(b2);
    	  Bond b3 = new org.openscience.cdk.Bond(a1, a4, 1.0);
    	  mol.addBond(b3);
    	  Bond b4 = new org.openscience.cdk.Bond(a2, a5, 1.0);
    	  mol.addBond(b4);
    	  Bond b5 = new org.openscience.cdk.Bond(a2, a6, 1.0);
    	  mol.addBond(b5);
    	  Bond b6 = new org.openscience.cdk.Bond(a6, a7, 1.0);
    	  mol.addBond(b6);
    	  Bond b7 = new org.openscience.cdk.Bond(a6, a8, 1.0);
    	  mol.addBond(b7);
    	  Bond b8 = new org.openscience.cdk.Bond(a8, a9, 2.0);
    	  mol.addBond(b8);
    	  Bond b9 = new org.openscience.cdk.Bond(a8, a10, 1.0);
    	  mol.addBond(b9);
    	  IBond b10 = new org.openscience.cdk.Bond(a10, a11, 1.0);
    	  mol.addBond(b10);
    	  IBond b11 = new org.openscience.cdk.Bond(a6, a12, 1.0);
    	  mol.addBond(b11);
    	  QueryAtomContainer q = QueryAtomContainerCreator.createBasicQueryContainer(mol);
  		  q.setID("ester of secondary alcohol attached to terminal vinyl group");
  		  return q;    	      	
    }
    public static IAtomContainer allylAlcohol() {
    	  IMolecule mol = new org.openscience.cdk.Molecule();
    	  Atom a1 = new Atom("C");
    	  mol.addAtom(a1);
    	  Atom a2 = new Atom("C");
    	  mol.addAtom(a2);
    	  Atom a3 = new Atom("C");
    	  mol.addAtom(a3);
    	  Atom a4 = new Atom("O");
    	  mol.addAtom(a4);
    	  Atom a5 = new Atom("H");
    	  mol.addAtom(a5);
    	  Atom a6 = new Atom("H");
    	  mol.addAtom(a6);
    	  Atom a7 = new Atom("H");
    	  mol.addAtom(a7);
    	  Atom a8 = new Atom("H");
    	  mol.addAtom(a8);
    	  Atom a9 = new Atom("H");
    	  mol.addAtom(a9);
    	  Atom a10 = new Atom("H");
    	  mol.addAtom(a10);
    	  Bond b1 = new org.openscience.cdk.Bond(a2, a1, 2.0);
    	  mol.addBond(b1);
    	  Bond b2 = new org.openscience.cdk.Bond(a3, a2, 1.0);
    	  mol.addBond(b2);
    	  Bond b3 = new org.openscience.cdk.Bond(a4, a3, 1.0);
    	  mol.addBond(b3);
    	  Bond b4 = new org.openscience.cdk.Bond(a1, a5, 1.0);
    	  mol.addBond(b4);
    	  Bond b5 = new org.openscience.cdk.Bond(a1, a6, 1.0);
    	  mol.addBond(b5);
    	  Bond b6 = new org.openscience.cdk.Bond(a2, a7, 1.0);
    	  mol.addBond(b6);
    	  Bond b7 = new org.openscience.cdk.Bond(a3, a8, 1.0);
    	  mol.addBond(b7);
    	  Bond b8 = new org.openscience.cdk.Bond(a3, a9, 1.0);
    	  mol.addBond(b8);
    	  Bond b9 = new org.openscience.cdk.Bond(a4, a10, 1.0);
    	  mol.addBond(b9);
    	  mol.setID("allyl alcohol");
    	  return mol;
    	//return createAtomContainer("C=CCO",);
    }        
    public static QueryAtomContainer allylAlcoholAcetal() {
    	//TODO "[*]C(OCC=C)(OCC=C)[H]"
    	//return createQuery("C(OCC=C)(OCC=C)[H]","allyl alcohol acetal");
    	  IMolecule mol = new org.openscience.cdk.Molecule();
    	  Atom a1 = new Atom("C");
    	  mol.addAtom(a1);
    	  Atom a2 = new Atom("O");
    	  mol.addAtom(a2);
    	  Atom a3 = new Atom("C");
    	  mol.addAtom(a3);
    	  Atom a4 = new Atom("C");
    	  mol.addAtom(a4);
    	  Atom a5 = new Atom("C");
    	  mol.addAtom(a5);
    	  Atom a6 = new Atom("O");
    	  mol.addAtom(a6);
    	  Atom a7 = new Atom("C");
    	  mol.addAtom(a7);
    	  Atom a8 = new Atom("C");
    	  mol.addAtom(a8);
    	  Atom a9 = new Atom("C");
    	  mol.addAtom(a9);
    	  Atom a10 = new Atom("H");
    	  mol.addAtom(a10);
    	  Atom a11 = new Atom("C");
    	  mol.addAtom(a11);
    	  Atom a12 = new Atom("H");
    	  mol.addAtom(a12);
    	  Atom a13 = new Atom("H");
    	  mol.addAtom(a13);
    	  Atom a14 = new Atom("H");
    	  mol.addAtom(a14);
    	  Atom a15 = new Atom("H");
    	  mol.addAtom(a15);
    	  Atom a16 = new Atom("H");
    	  mol.addAtom(a16);
    	  Atom a17 = new Atom("H");
    	  mol.addAtom(a17);
    	  Atom a18 = new Atom("H");
    	  mol.addAtom(a18);
    	  Atom a19 = new Atom("H");
    	  mol.addAtom(a19);
    	  Atom a20 = new Atom("H");
    	  mol.addAtom(a20);
    	  Atom a21 = new Atom("H");
    	  mol.addAtom(a21);
    	  Bond b1 = new org.openscience.cdk.Bond(a2, a1, 1.0);
    	  mol.addBond(b1);
    	  Bond b2 = new org.openscience.cdk.Bond(a3, a2, 1.0);
    	  mol.addBond(b2);
    	  Bond b3 = new org.openscience.cdk.Bond(a4, a3, 1.0);
    	  mol.addBond(b3);
    	  Bond b4 = new org.openscience.cdk.Bond(a5, a4, 2.0);
    	  mol.addBond(b4);
    	  Bond b5 = new org.openscience.cdk.Bond(a6, a1, 1.0);
    	  mol.addBond(b5);
    	  Bond b6 = new org.openscience.cdk.Bond(a7, a6, 1.0);
    	  mol.addBond(b6);
    	  Bond b7 = new org.openscience.cdk.Bond(a8, a7, 1.0);
    	  mol.addBond(b7);
    	  Bond b8 = new org.openscience.cdk.Bond(a9, a8, 2.0);
    	  mol.addBond(b8);
    	  Bond b9 = new org.openscience.cdk.Bond(a10, a1, 1.0);
    	  mol.addBond(b9);
    	  Bond b10 = new org.openscience.cdk.Bond(a1, a11, 1.0);
    	  mol.addBond(b10);
    	  Bond b11 = new org.openscience.cdk.Bond(a3, a12, 1.0);
    	  mol.addBond(b11);
    	  Bond b12 = new org.openscience.cdk.Bond(a3, a13, 1.0);
    	  mol.addBond(b12);
    	  Bond b13 = new org.openscience.cdk.Bond(a7, a14, 1.0);
    	  mol.addBond(b13);
    	  Bond b14 = new org.openscience.cdk.Bond(a7, a15, 1.0);
    	  mol.addBond(b14);
    	  Bond b15 = new org.openscience.cdk.Bond(a4, a16, 1.0);
    	  mol.addBond(b15);
    	  Bond b16 = new org.openscience.cdk.Bond(a8, a17, 1.0);
    	  mol.addBond(b16);
    	  Bond b17 = new org.openscience.cdk.Bond(a5, a18, 1.0);
    	  mol.addBond(b17);
    	  Bond b18 = new org.openscience.cdk.Bond(a5, a19, 1.0);
    	  mol.addBond(b18);
    	  Bond b19 = new org.openscience.cdk.Bond(a9, a20, 1.0);
    	  mol.addBond(b19);
    	  Bond b20 = new org.openscience.cdk.Bond(a9, a21, 1.0);
    	  mol.addBond(b20);
    	  QueryAtomContainer q = QueryAtomContainerCreator.createBasicQueryContainer(mol);
  		  q.setID("allyl alcohol acetal");
  		  return q;    	      	
    	
    }
    public static QueryAtomContainer allylAlcoholEsterDerivative() {
    	//TODO "CC=CCOC(=O)C[*]"
    	  Molecule mol = new org.openscience.cdk.Molecule();
    	  Atom a1 = new Atom("O");
    	  mol.addAtom(a1);
    	  Atom a2 = new Atom("C");
    	  mol.addAtom(a2);
    	  Atom a3 = new Atom("C");
    	  mol.addAtom(a3);
    	  Atom a4 = new Atom("O");
    	  mol.addAtom(a4);
    	  Atom a5 = new Atom("C");
    	  mol.addAtom(a5);
    	  Atom a6 = new Atom("H");
    	  mol.addAtom(a6);
    	  Atom a7 = new Atom("H");
    	  mol.addAtom(a7);
    	  Atom a8 = new Atom("C");
    	  mol.addAtom(a8);
    	  Atom a9 = new Atom("H");
    	  mol.addAtom(a9);
    	  Atom a10 = new Atom("C");
    	  mol.addAtom(a10);
    	  Atom a11 = new Atom("H");
    	  mol.addAtom(a11);
    	  Atom a12 = new Atom("H");
    	  mol.addAtom(a12);
    	  Bond b1 = new org.openscience.cdk.Bond(a2, a1, 2.0);
    	  mol.addBond(b1);
    	  Bond b2 = new org.openscience.cdk.Bond(a3, a2, 1.0);
    	  mol.addBond(b2);
    	  Bond b3 = new org.openscience.cdk.Bond(a4, a2, 1.0);
    	  mol.addBond(b3);
    	  Bond b4 = new org.openscience.cdk.Bond(a5, a4, 1.0);
    	  mol.addBond(b4);
    	  Bond b5 = new org.openscience.cdk.Bond(a6, a5, 1.0);
    	  mol.addBond(b5);
    	  Bond b6 = new org.openscience.cdk.Bond(a7, a5, 1.0);
    	  mol.addBond(b6);
    	  Bond b7 = new org.openscience.cdk.Bond(a8, a5, 1.0);
    	  mol.addBond(b7);
    	  Bond b8 = new org.openscience.cdk.Bond(a9, a8, 1.0);
    	  mol.addBond(b8);
    	  Bond b9 = new org.openscience.cdk.Bond(a10, a8, 2.0);
    	  mol.addBond(b9);
    	  Bond b10 = new org.openscience.cdk.Bond(a11, a10, 1.0);
    	  mol.addBond(b10);
    	  Bond b11 = new org.openscience.cdk.Bond(a12, a10, 1.0);
    	  mol.addBond(b11);
    	  QueryAtomContainer q = QueryAtomContainerCreator.createBasicQueryContainer(mol);
  		  q.setID("allyl alcohol ester derivative");
  		  return q;
    	
    	//return createQuery("O=C(C)OC([H])([H])C([H])=C([H])[H]","allyl alcohol ester derivative");
    }    
    public static IAtomContainer allylMercaptan() {
    	//return createAtomContainer("[H]SC([H])([H])C([H])=C([H])[H]","allyl mercaptan");
    	  IMolecule mol = new org.openscience.cdk.Molecule();
    	  Atom a1 = new Atom("C");
    	  mol.addAtom(a1);
    	  Atom a2 = new Atom("C");
    	  mol.addAtom(a2);
    	  Atom a3 = new Atom("C");
    	  mol.addAtom(a3);
    	  Atom a4 = new Atom("S");
    	  mol.addAtom(a4);
    	  Atom a5 = new Atom("H");
    	  mol.addAtom(a5);
    	  Atom a6 = new Atom("H");
    	  mol.addAtom(a6);
    	  Atom a7 = new Atom("H");
    	  mol.addAtom(a7);
    	  Atom a8 = new Atom("H");
    	  mol.addAtom(a8);
    	  Atom a9 = new Atom("H");
    	  mol.addAtom(a9);
    	  Atom a10 = new Atom("H");
    	  mol.addAtom(a10);
    	  Bond b1 = new org.openscience.cdk.Bond(a2, a1, 2.0);
    	  mol.addBond(b1);
    	  Bond b2 = new org.openscience.cdk.Bond(a3, a2, 1.0);
    	  mol.addBond(b2);
    	  Bond b3 = new org.openscience.cdk.Bond(a4, a3, 1.0);
    	  mol.addBond(b3);
    	  Bond b4 = new org.openscience.cdk.Bond(a1, a5, 1.0);
    	  mol.addBond(b4);
    	  Bond b5 = new org.openscience.cdk.Bond(a1, a6, 1.0);
    	  mol.addBond(b5);
    	  Bond b6 = new org.openscience.cdk.Bond(a2, a7, 1.0);
    	  mol.addBond(b6);
    	  Bond b7 = new org.openscience.cdk.Bond(a3, a8, 1.0);
    	  mol.addBond(b7);
    	  Bond b8 = new org.openscience.cdk.Bond(a3, a9, 1.0);
    	  mol.addBond(b8);
    	  Bond b9 = new org.openscience.cdk.Bond(a4, a10, 1.0);
    	  mol.addBond(b9);
    	  mol.setID("allyl mercaptan");
    	  return mol;
    }         
    public static IAtomContainer allylAmine() {
//    	return createAtomContainer("C=CCN","allyl amine");
    	  IMolecule mol = new org.openscience.cdk.Molecule();
    	  Atom a1 = new Atom("C");
    	  mol.addAtom(a1);
    	  Atom a2 = new Atom("C");
    	  mol.addAtom(a2);
    	  Atom a3 = new Atom("C");
    	  mol.addAtom(a3);
    	  Atom a4 = new Atom("N");
    	  mol.addAtom(a4);
    	  Atom a5 = new Atom("H");
    	  mol.addAtom(a5);
    	  Atom a6 = new Atom("H");
    	  mol.addAtom(a6);
    	  Atom a7 = new Atom("H");
    	  mol.addAtom(a7);
    	  Atom a8 = new Atom("H");
    	  mol.addAtom(a8);
    	  Atom a9 = new Atom("H");
    	  mol.addAtom(a9);
    	  Atom a10 = new Atom("H");
    	  mol.addAtom(a10);
    	  Atom a11 = new Atom("H");
    	  mol.addAtom(a11);
    	  Bond b1 = new org.openscience.cdk.Bond(a2, a1, 2.0);
    	  mol.addBond(b1);
    	  Bond b2 = new org.openscience.cdk.Bond(a3, a2, 1.0);
    	  mol.addBond(b2);
    	  Bond b3 = new org.openscience.cdk.Bond(a4, a3, 1.0);
    	  mol.addBond(b3);
    	  Bond b4 = new org.openscience.cdk.Bond(a1, a5, 1.0);
    	  mol.addBond(b4);
    	  Bond b5 = new org.openscience.cdk.Bond(a1, a6, 1.0);
    	  mol.addBond(b5);
    	  Bond b6 = new org.openscience.cdk.Bond(a2, a7, 1.0);
    	  mol.addBond(b6);
    	  Bond b7 = new org.openscience.cdk.Bond(a3, a8, 1.0);
    	  mol.addBond(b7);
    	  Bond b8 = new org.openscience.cdk.Bond(a3, a9, 1.0);
    	  mol.addBond(b8);
    	  Bond b9 = new org.openscience.cdk.Bond(a4, a10, 1.0);
    	  mol.addBond(b9);
    	  Bond b10 = new org.openscience.cdk.Bond(a4, a11, 1.0);
    	  mol.addBond(b10);
    	  mol.setID("allyl amine");
    	  return mol;
    }         
    public static IAtomContainer allylSulphide() {
    	//return createAtomContainer("C=CCSCC=C","allyl sulphide");
    	  Molecule mol = new org.openscience.cdk.Molecule();
    	  Atom a1 = new Atom("C");
    	  mol.addAtom(a1);
    	  Atom a2 = new Atom("C");
    	  mol.addAtom(a2);
    	  Atom a3 = new Atom("C");
    	  mol.addAtom(a3);
    	  Atom a4 = new Atom("S");
    	  mol.addAtom(a4);
    	  Atom a5 = new Atom("C");
    	  mol.addAtom(a5);
    	  Atom a6 = new Atom("C");
    	  mol.addAtom(a6);
    	  Atom a7 = new Atom("C");
    	  mol.addAtom(a7);
    	  Atom a8 = new Atom("H");
    	  mol.addAtom(a8);
    	  Atom a9 = new Atom("H");
    	  mol.addAtom(a9);
    	  Atom a10 = new Atom("H");
    	  mol.addAtom(a10);
    	  Atom a11 = new Atom("H");
    	  mol.addAtom(a11);
    	  Atom a12 = new Atom("H");
    	  mol.addAtom(a12);
    	  Atom a13 = new Atom("H");
    	  mol.addAtom(a13);
    	  Atom a14 = new Atom("H");
    	  mol.addAtom(a14);
    	  Atom a15 = new Atom("H");
    	  mol.addAtom(a15);
    	  Atom a16 = new Atom("H");
    	  mol.addAtom(a16);
    	  Atom a17 = new Atom("H");
    	  mol.addAtom(a17);
    	  Bond b1 = new org.openscience.cdk.Bond(a2, a1, 2.0);
    	  mol.addBond(b1);
    	  Bond b2 = new org.openscience.cdk.Bond(a3, a2, 1.0);
    	  mol.addBond(b2);
    	  Bond b3 = new org.openscience.cdk.Bond(a4, a3, 1.0);
    	  mol.addBond(b3);
    	  Bond b4 = new org.openscience.cdk.Bond(a5, a4, 1.0);
    	  mol.addBond(b4);
    	  Bond b5 = new org.openscience.cdk.Bond(a6, a5, 1.0);
    	  mol.addBond(b5);
    	  Bond b6 = new org.openscience.cdk.Bond(a7, a6, 2.0);
    	  mol.addBond(b6);
    	  Bond b7 = new org.openscience.cdk.Bond(a1, a8, 1.0);
    	  mol.addBond(b7);
    	  Bond b8 = new org.openscience.cdk.Bond(a1, a9, 1.0);
    	  mol.addBond(b8);
    	  Bond b9 = new org.openscience.cdk.Bond(a2, a10, 1.0);
    	  mol.addBond(b9);
    	  Bond b10 = new org.openscience.cdk.Bond(a3, a11, 1.0);
    	  mol.addBond(b10);
    	  Bond b11 = new org.openscience.cdk.Bond(a3, a12, 1.0);
    	  mol.addBond(b11);
    	  Bond b12 = new org.openscience.cdk.Bond(a5, a13, 1.0);
    	  mol.addBond(b12);
    	  Bond b13 = new org.openscience.cdk.Bond(a5, a14, 1.0);
    	  mol.addBond(b13);
    	  Bond b14 = new org.openscience.cdk.Bond(a6, a15, 1.0);
    	  mol.addBond(b14);
    	  Bond b15 = new org.openscience.cdk.Bond(a7, a16, 1.0);
    	  mol.addBond(b15);
    	  Bond b16 = new org.openscience.cdk.Bond(a7, a17, 1.0);
    	  mol.addBond(b16);
    	  mol.setID("allyl sulphide");
    	  return mol;
    }         
    public static IAtomContainer allylThioester() {
    	//	return createQuery("C(=O)SCC=C","allyl thioester");
    	  Molecule mol = new org.openscience.cdk.Molecule();
    	  Atom a1 = new Atom("C");
    	  mol.addAtom(a1);
    	  Atom a2 = new Atom("O");
    	  mol.addAtom(a2);
    	  Atom a3 = new Atom("S");
    	  mol.addAtom(a3);
    	  Atom a4 = new Atom("C");
    	  mol.addAtom(a4);
    	  Atom a5 = new Atom("C");
    	  mol.addAtom(a5);
    	  Atom a6 = new Atom("C");
    	  mol.addAtom(a6);
    	  Atom a7 = new Atom("H");
    	  mol.addAtom(a7);
    	  Atom a8 = new Atom("H");
    	  mol.addAtom(a8);
    	  Atom a9 = new Atom("H");
    	  mol.addAtom(a9);
    	  Atom a10 = new Atom("H");
    	  mol.addAtom(a10);
    	  Atom a11 = new Atom("H");
    	  mol.addAtom(a11);
    	  Atom a12 = new Atom("H");
    	  mol.addAtom(a12);
    	  Bond b1 = new org.openscience.cdk.Bond(a2, a1, 2.0);
    	  mol.addBond(b1);
    	  Bond b2 = new org.openscience.cdk.Bond(a3, a1, 1.0);
    	  mol.addBond(b2);
    	  Bond b3 = new org.openscience.cdk.Bond(a4, a3, 1.0);
    	  mol.addBond(b3);
    	  Bond b4 = new org.openscience.cdk.Bond(a5, a4, 1.0);
    	  mol.addBond(b4);
    	  Bond b5 = new org.openscience.cdk.Bond(a6, a5, 2.0);
    	  mol.addBond(b5);
    	  Bond b6 = new org.openscience.cdk.Bond(a1, a7, 1.0);
    	  mol.addBond(b6);
    	  Bond b7 = new org.openscience.cdk.Bond(a4, a8, 1.0);
    	  mol.addBond(b7);
    	  Bond b8 = new org.openscience.cdk.Bond(a4, a9, 1.0);
    	  mol.addBond(b8);
    	  Bond b9 = new org.openscience.cdk.Bond(a5, a10, 1.0);
    	  mol.addBond(b9);
    	  Bond b10 = new org.openscience.cdk.Bond(a6, a11, 1.0);
    	  mol.addBond(b10);
    	  Bond b11 = new org.openscience.cdk.Bond(a6, a12, 1.0);
    	  mol.addBond(b11);
    	  mol.setID("allyl thioester");
    	  return mol;
    }         
    public static IAtomContainer acrolein() {
    	//return createAtomContainer("C=CC=O","acrolein");
    	  Molecule mol = new org.openscience.cdk.Molecule();
    	  Atom a1 = new Atom("C");
    	  mol.addAtom(a1);
    	  Atom a2 = new Atom("C");
    	  mol.addAtom(a2);
    	  Atom a3 = new Atom("C");
    	  mol.addAtom(a3);
    	  Atom a4 = new Atom("O");
    	  mol.addAtom(a4);
    	  Atom a5 = new Atom("H");
    	  mol.addAtom(a5);
    	  Atom a6 = new Atom("H");
    	  mol.addAtom(a6);
    	  Atom a7 = new Atom("H");
    	  mol.addAtom(a7);
    	  Atom a8 = new Atom("H");
    	  mol.addAtom(a8);
    	  Bond b1 = new org.openscience.cdk.Bond(a2, a1, 2.0);
    	  mol.addBond(b1);
    	  Bond b2 = new org.openscience.cdk.Bond(a3, a2, 1.0);
    	  mol.addBond(b2);
    	  Bond b3 = new org.openscience.cdk.Bond(a4, a3, 2.0);
    	  mol.addBond(b3);
    	  Bond b4 = new org.openscience.cdk.Bond(a1, a5, 1.0);
    	  mol.addBond(b4);
    	  Bond b5 = new org.openscience.cdk.Bond(a1, a6, 1.0);
    	  mol.addBond(b5);
    	  Bond b6 = new org.openscience.cdk.Bond(a2, a7, 1.0);
    	  mol.addBond(b6);
    	  Bond b7 = new org.openscience.cdk.Bond(a3, a8, 1.0);
    	  mol.addBond(b7);
    	  mol.setID("acrolein");
    	  return mol;
    }       
    public static IAtomContainer methacrolein() {
    	//return createAtomContainer("C=C(C)C=O","methacrolein");
    	  Molecule mol = new org.openscience.cdk.Molecule();
    	  Atom a1 = new Atom("C");
    	  mol.addAtom(a1);
    	  Atom a2 = new Atom("C");
    	  mol.addAtom(a2);
    	  Atom a3 = new Atom("C");
    	  mol.addAtom(a3);
    	  Atom a4 = new Atom("C");
    	  mol.addAtom(a4);
    	  Atom a5 = new Atom("O");
    	  mol.addAtom(a5);
    	  Atom a6 = new Atom("H");
    	  mol.addAtom(a6);
    	  Atom a7 = new Atom("H");
    	  mol.addAtom(a7);
    	  Atom a8 = new Atom("H");
    	  mol.addAtom(a8);
    	  Atom a9 = new Atom("H");
    	  mol.addAtom(a9);
    	  Atom a10 = new Atom("H");
    	  mol.addAtom(a10);
    	  Atom a11 = new Atom("H");
    	  mol.addAtom(a11);
    	  Bond b1 = new org.openscience.cdk.Bond(a2, a1, 2.0);
    	  mol.addBond(b1);
    	  Bond b2 = new org.openscience.cdk.Bond(a3, a2, 1.0);
    	  mol.addBond(b2);
    	  Bond b3 = new org.openscience.cdk.Bond(a4, a2, 1.0);
    	  mol.addBond(b3);
    	  Bond b4 = new org.openscience.cdk.Bond(a5, a4, 2.0);
    	  mol.addBond(b4);
    	  Bond b5 = new org.openscience.cdk.Bond(a1, a6, 1.0);
    	  mol.addBond(b5);
    	  Bond b6 = new org.openscience.cdk.Bond(a1, a7, 1.0);
    	  mol.addBond(b6);
    	  Bond b7 = new org.openscience.cdk.Bond(a3, a8, 1.0);
    	  mol.addBond(b7);
    	  Bond b8 = new org.openscience.cdk.Bond(a3, a9, 1.0);
    	  mol.addBond(b8);
    	  Bond b9 = new org.openscience.cdk.Bond(a3, a10, 1.0);
    	  mol.addBond(b9);
    	  Bond b10 = new org.openscience.cdk.Bond(a4, a11, 1.0);
    	  mol.addBond(b10);
    	  mol.setID("Methacrolein");
    	  return mol;
    }       
    public static IAtomContainer methacroleinAcetal() {
    	  Molecule mol = new org.openscience.cdk.Molecule();
    	  Atom a1 = new Atom("C");
    	  mol.addAtom(a1);
    	  Atom a2 = new Atom("C");
    	  mol.addAtom(a2);
    	  Atom a3 = new Atom("C");
    	  mol.addAtom(a3);
    	  Atom a4 = new Atom("C");
    	  mol.addAtom(a4);
    	  Atom a5 = new Atom("O");
    	  mol.addAtom(a5);
    	  Atom a6 = new Atom("O");
    	  mol.addAtom(a6);
    	  Atom a7 = new Atom("H");
    	  mol.addAtom(a7);
    	  Atom a8 = new Atom("H");
    	  mol.addAtom(a8);
    	  Atom a9 = new Atom("H");
    	  mol.addAtom(a9);
    	  Atom a10 = new Atom("H");
    	  mol.addAtom(a10);
    	  Atom a11 = new Atom("C");
    	  mol.addAtom(a11);
    	  Atom a12 = new Atom("C");
    	  mol.addAtom(a12);
    	  Atom a13 = new Atom("H");
    	  mol.addAtom(a13);
    	  Atom a14 = new Atom("H");
    	  mol.addAtom(a14);
    	  Atom a15 = new Atom("H");
    	  mol.addAtom(a15);
    	  Atom a16 = new Atom("H");
    	  mol.addAtom(a16);
    	  Atom a17 = new Atom("H");
    	  mol.addAtom(a17);
    	  Atom a18 = new Atom("H");
    	  mol.addAtom(a18);
    	  Atom a19 = new Atom("H");
    	  mol.addAtom(a19);
    	  Atom a20 = new Atom("H");
    	  mol.addAtom(a20);
    	  Bond b1 = new org.openscience.cdk.Bond(a1, a2, 1.0);
    	  mol.addBond(b1);
    	  Bond b2 = new org.openscience.cdk.Bond(a2, a3, 1.0);
    	  mol.addBond(b2);
    	  Bond b3 = new org.openscience.cdk.Bond(a3, a4, 2.0);
    	  mol.addBond(b3);
    	  Bond b4 = new org.openscience.cdk.Bond(a2, a5, 1.0);
    	  mol.addBond(b4);
    	  Bond b5 = new org.openscience.cdk.Bond(a2, a6, 1.0);
    	  mol.addBond(b5);
    	  Bond b6 = new org.openscience.cdk.Bond(a1, a7, 1.0);
    	  mol.addBond(b6);
    	  Bond b7 = new org.openscience.cdk.Bond(a1, a8, 1.0);
    	  mol.addBond(b7);
    	  Bond b8 = new org.openscience.cdk.Bond(a1, a9, 1.0);
    	  mol.addBond(b8);
    	  Bond b9 = new org.openscience.cdk.Bond(a3, a10, 1.0);
    	  mol.addBond(b9);
    	  Bond b10 = new org.openscience.cdk.Bond(a5, a11, 1.0);
    	  mol.addBond(b10);
    	  Bond b11 = new org.openscience.cdk.Bond(a6, a12, 1.0);
    	  mol.addBond(b11);
    	  Bond b12 = new org.openscience.cdk.Bond(a4, a13, 1.0);
    	  mol.addBond(b12);
    	  Bond b13 = new org.openscience.cdk.Bond(a4, a14, 1.0);
    	  mol.addBond(b13);
    	  Bond b14 = new org.openscience.cdk.Bond(a12, a15, 1.0);
    	  mol.addBond(b14);
    	  Bond b15 = new org.openscience.cdk.Bond(a12, a16, 1.0);
    	  mol.addBond(b15);
    	  Bond b16 = new org.openscience.cdk.Bond(a12, a17, 1.0);
    	  mol.addBond(b16);
    	  Bond b17 = new org.openscience.cdk.Bond(a11, a18, 1.0);
    	  mol.addBond(b17);
    	  Bond b18 = new org.openscience.cdk.Bond(a11, a19, 1.0);
    	  mol.addBond(b18);
    	  Bond b19 = new org.openscience.cdk.Bond(a11, a20, 1.0);
    	  mol.addBond(b19);
    	  mol.setID("Methacrolein acetal");
    	  return mol;    	  
    	
    }
    public static IAtomContainer acrylicAcid() {
    	//return createAtomContainer("C=CC(O)=O","acrylic acid");
    	  Molecule mol = new org.openscience.cdk.Molecule();
    	  Atom a1 = new Atom("C");
    	  mol.addAtom(a1);
    	  Atom a2 = new Atom("C");
    	  mol.addAtom(a2);
    	  Atom a3 = new Atom("C");
    	  mol.addAtom(a3);
    	  Atom a4 = new Atom("O");
    	  mol.addAtom(a4);
    	  Atom a5 = new Atom("O");
    	  mol.addAtom(a5);
    	  Atom a6 = new Atom("H");
    	  mol.addAtom(a6);
    	  Atom a7 = new Atom("H");
    	  mol.addAtom(a7);
    	  Atom a8 = new Atom("H");
    	  mol.addAtom(a8);
    	  Atom a9 = new Atom("H");
    	  mol.addAtom(a9);
    	  Bond b1 = new org.openscience.cdk.Bond(a2, a1, 2.0);
    	  mol.addBond(b1);
    	  Bond b2 = new org.openscience.cdk.Bond(a3, a2, 1.0);
    	  mol.addBond(b2);
    	  Bond b3 = new org.openscience.cdk.Bond(a4, a3, 1.0);
    	  mol.addBond(b3);
    	  Bond b4 = new org.openscience.cdk.Bond(a5, a3, 2.0);
    	  mol.addBond(b4);
    	  Bond b5 = new org.openscience.cdk.Bond(a1, a6, 1.0);
    	  mol.addBond(b5);
    	  Bond b6 = new org.openscience.cdk.Bond(a1, a7, 1.0);
    	  mol.addBond(b6);
    	  Bond b7 = new org.openscience.cdk.Bond(a2, a8, 1.0);
    	  mol.addBond(b7);
    	  Bond b8 = new org.openscience.cdk.Bond(a4, a9, 1.0);
    	  mol.addBond(b8);
    	  mol.setID("acrylic acid");
    	  return mol;
    }       
    public static IAtomContainer methacrylicAcid() {
    	//return createAtomContainer("C=C(C)C(O)=O","methacrylic acid");
    	  Molecule mol = new org.openscience.cdk.Molecule();
    	  Atom a1 = new Atom("C");
    	  mol.addAtom(a1);
    	  Atom a2 = new Atom("C");
    	  mol.addAtom(a2);
    	  Atom a3 = new Atom("C");
    	  mol.addAtom(a3);
    	  Atom a4 = new Atom("C");
    	  mol.addAtom(a4);
    	  Atom a5 = new Atom("O");
    	  mol.addAtom(a5);
    	  Atom a6 = new Atom("O");
    	  mol.addAtom(a6);
    	  Atom a7 = new Atom("H");
    	  mol.addAtom(a7);
    	  Atom a8 = new Atom("H");
    	  mol.addAtom(a8);
    	  Atom a9 = new Atom("H");
    	  mol.addAtom(a9);
    	  Atom a10 = new Atom("H");
    	  mol.addAtom(a10);
    	  Atom a11 = new Atom("H");
    	  mol.addAtom(a11);
    	  Atom a12 = new Atom("H");
    	  mol.addAtom(a12);
    	  Bond b1 = new org.openscience.cdk.Bond(a2, a1, 2.0);
    	  mol.addBond(b1);
    	  Bond b2 = new org.openscience.cdk.Bond(a3, a2, 1.0);
    	  mol.addBond(b2);
    	  Bond b3 = new org.openscience.cdk.Bond(a4, a2, 1.0);
    	  mol.addBond(b3);
    	  Bond b4 = new org.openscience.cdk.Bond(a5, a4, 1.0);
    	  mol.addBond(b4);
    	  Bond b5 = new org.openscience.cdk.Bond(a6, a4, 2.0);
    	  mol.addBond(b5);
    	  Bond b6 = new org.openscience.cdk.Bond(a1, a7, 1.0);
    	  mol.addBond(b6);
    	  Bond b7 = new org.openscience.cdk.Bond(a1, a8, 1.0);
    	  mol.addBond(b7);
    	  Bond b8 = new org.openscience.cdk.Bond(a3, a9, 1.0);
    	  mol.addBond(b8);
    	  Bond b9 = new org.openscience.cdk.Bond(a3, a10, 1.0);
    	  mol.addBond(b9);
    	  Bond b10 = new org.openscience.cdk.Bond(a3, a11, 1.0);
    	  mol.addBond(b10);
    	  Bond b11 = new org.openscience.cdk.Bond(a5, a12, 1.0);
    	  mol.addBond(b11);
    	  mol.setID("methacrylic acid");
    	  return mol;
    }    
    public static boolean isAcetylenic(IAtomContainer  mol) {
    	for (int i = 0; i < mol.getBondCount(); i++) { 
    		IBond b = mol.getBondAt(i);
    		if ((b.getOrder() == CDKConstants.BONDORDER_TRIPLE) &&
    			 (b.getAtomAt(0).getSymbol().equals("C")) &&
				 (b.getAtomAt(1).getSymbol().equals("C")) 
    		   ) {
    			logger.info("Acetylenic group found");
    			return true;		
    		}
    	}
    	return false;
    }
    
    public static QueryAtomContainer stericallyHindered() {

	  	  QueryAtomContainer mol = new QueryAtomContainer();
		  SymbolQueryAtom a[] = new SymbolQueryAtom[6];
	
		  InverseSymbolSetQueryAtom[] r = new InverseSymbolSetQueryAtom[6];
		  for (int i=0; i < 6; i++) {
		  	r[i] = new InverseSymbolSetQueryAtom();
			a[i] = new SymbolQueryAtom(new Atom("C"));
			mol.addAtom(a[i]);
		  	r[i].addSymbol("H");
		  	mol.addAtom(r[i]);
		  	if (i>0) {
		  		AnyOrderQueryBond b = new AnyOrderQueryBond(a[i], a[i-1],1);
		  		mol.addBond(b);
		  	}
		  }
		  SymbolQueryAtom a7 = new SymbolQueryAtom(new Atom("C"));
    	  mol.addAtom(a7);
		  SymbolQueryAtom a11 = new SymbolQueryAtom(new Atom("C"));
    	  mol.addAtom(a11);

		  InverseSymbolSetQueryAtom hindered = new InverseSymbolSetQueryAtom();
		  hindered.addSymbol("H"); //anything but H
		  //hindered.addSymbol("C");
    	  mol.addAtom(hindered);    	  
	  	  AnyOrderQueryBond b = new AnyOrderQueryBond(a[3], hindered,1);
	  	  mol.addBond(b);
    	  
    	  OrderQueryBond b7 = new OrderQueryBond(a[2], a7, 1.0);
    	  mol.addBond(b7);
    	  OrderQueryBond b8 = new OrderQueryBond(a7, r[0], 1.0);
    	  mol.addBond(b8);
    	  OrderQueryBond b9 = new OrderQueryBond(a7, r[1], 1.0);
    	  mol.addBond(b9);
    	  OrderQueryBond b10 = new OrderQueryBond(a7, r[2], 1.0);
    	  mol.addBond(b10);
    	  OrderQueryBond b11 = new OrderQueryBond(a[4], a11, 1.0);
    	  mol.addBond(b11);
    	  OrderQueryBond b12 = new OrderQueryBond(a11, r[3], 1.0);
    	  mol.addBond(b12);
    	  OrderQueryBond b13 = new OrderQueryBond(a11, r[4], 1.0);
    	  mol.addBond(b13);
    	  OrderQueryBond b14 = new OrderQueryBond(a11, r[5], 1.0);
    	  mol.addBond(b14);
    	  
      	  mol.setID("Sterically hindered\t");
      	  return mol;
    }
    public static boolean hasSubstituents(IAtomContainer mol, IRingSet rings) {
        
    	if ((rings == null) || (rings.getAtomContainerCount()==0)) return false;
    	int size = rings.getAtomContainerCount();
    	Ring r = null;
    	//Atom[] neighbors = null;
    	IBond[] neighbors = null;
	    for (int i=0; i < size; i++) {
	        r = (Ring) rings.getAtomContainer(i);
	        for (int j=0; j< r.getAtomCount(); j++) {
	        	//neighbors = mol.getConnectedAtoms(r.getAtomAt(j));
	        	neighbors = mol.getConnectedBonds(r.getAtomAt(j));
	        	for (int n=0; n< neighbors.length; n++) {
	        		if (neighbors[n].getAtomAt(0).getSymbol().equals("H")) continue; 
	        		if (neighbors[n].getAtomAt(1).getSymbol().equals("H")) continue;
	        		//if there is acyclic bond then it is substituent
	        		if (!neighbors[n].getFlag(CDKConstants.ISINRING)) return true;
	        		/*
	        		if (!r.contains(neighbors[n])) {
	        			logger.debug("Ring has substituent(s)\t","YES\t",neighbors[n].getSymbol());
	        			return true; 
	        		}
	        		*/
	        	}
	        }
	    }
	    logger.debug("Ring with substituent(s)\t","NO");
	    return false;
    }
    public static boolean isCommonTerpene(IAtomContainer  mol) {
    	SSSRFinder ssrf = new SSSRFinder(mol);
    	return isCommonTerpene(mol,ssrf.findSSSR());
    }
    public static boolean isCommonTerpene(IAtomContainer  mol, IRingSet rings) {
    	
    	MFAnalyser mf = new MFAnalyser(mol);
    	int c = mf.getAtomCount("C");
    	
    	if ((c % 5) != 0) return false;
    	int n = c / 5;
    	if (n < 2) return false;
    	int h =mf.getAtomCount("H");
    	
    	if (h/8  != n)  {
    		logger.debug(mf.getFormulaHashtable());
    		return false;
    	}
    	if ((c + h + mf.getAtomCount("O")) < mol.getAtomCount()) return false; //athoms different than C,H,O
    	
    	QueryAtomContainer q = isopreneUnit();
		List list = FunctionalGroups.getBondMap(mol, q,false);
		if ((list == null) || (list.size() == 0)) {
			logger.debug("NO isoprene unit found");
			return false;
		}
		FunctionalGroups.markMaps(mol, q, list);
		
		ArrayList ids = new ArrayList();
		ids.add(q.getID());
		
		int size = 0;
		if (rings != null) size = rings.getAtomContainerCount();
		if ((rings!=null) && (size > 0) && ((c + h) == mol.getAtomCount())) {
//			w monociklicnite ima 2 double bond, w biciklicnite ima 1 double
			int db = 0;
			for (int i=0; i< mol.getBondCount(); i++)
				if (mol.getBondAt(i).getOrder() == CDKConstants.BONDORDER_DOUBLE) db++;
			if ((size == 1) && (db != 2)) {
				logger.debug("Single ring , double bonds ",db);
				return false;
			}
			if ((size == 2) && (db != 1)) {
				logger.debug("Two rings , double bonds ",db);
				return false;
			}
			logger.debug("May be terpene");
			//za ostanalite neznam
		} 
		
		if  (mf.getAtomCount("O")>0) { //check for alcohol, aldehyde, carboxylic acid
			preProcess(mol);
			q= alcohol(false); ids.add(q.getID());
			list = FunctionalGroups.getBondMap(mol, q,true);
			FunctionalGroups.markMaps(mol, q, list);
			q= aldehyde(); ids.add(q.getID());
			list = FunctionalGroups.getBondMap(mol, q,true);
			FunctionalGroups.markMaps(mol, q, list);
			q= carboxylicAcid(); ids.add(q.getID());
			list = FunctionalGroups.getBondMap(mol, q,true);
			FunctionalGroups.markMaps(mol, q, list);			
		}
		
		boolean b = hasMarkedOnlyTheseGroups(mol,ids);
		if (logger.isDebugEnabled()) {
			logger.debug(mapToString(mol));
			if (b) logger.debug("This compound is a terpene (terpenoid)");
			else  logger.debug("This compound is NOT a terpene");
		}
		return b;

     }
	public static boolean singleFusedRing(IAtomContainer  mol, IRingSet rings) {
	    
		if ((rings == null) || (rings.getAtomContainerCount() ==0) ) return false;
		int size = rings.getAtomContainerCount();
		if (size==1) return true;
		List allRings = rings.getConnectedRings((Ring)rings.getAtomContainer(0));
		boolean b = (allRings.size() == size);
		if (b) logger.debug("Single fused ring found");
		allRings = null;
		return b;
	}
	public static boolean hasGroup(IAtomContainer mol, IAtomContainer q) {
		//search for these groups relies on CHn marks set by markCHn 
		return hasGroup(mol,q,needsPreprocessing(q));
	}
	public static boolean hasGroup(IAtomContainer mol, IAtomContainer q, boolean preprocess) {
		if (q.getAtomCount() > mol.getAtomCount()) {
			logger.debug("A query with more atoms than a molecule!");
			return false;
		}
		if (preprocess) {
			logger.debug("Marking CHn groups");
			preProcess(mol);
		}
		try {
			if (UniversalIsomorphismTester.isSubgraph(mol,q)) {
				//logger.debug(mapToString(mol));
				logger.info("Molecule \t",mol.getID(),MSG_HASGROUP,q.getID(),"\tYES");
				return true;
			} else {
				//logger.debug(mapToString(mol));
				logger.debug("Molecule \t",mol.getID(),MSG_HASGROUP,q.getID(),"\tNO");
				return false;		 
			}
		} catch (Exception x) {
			logger.error(x);
			return false;
		} 
	}
	public static boolean isSubstance(IAtomContainer mol, IAtomContainer q) {
		try {
		if (mol.getAtomCount() != q.getAtomCount()) return false; // no need to spend time on isomorphism ...
		if (mol.getBondCount() != q.getBondCount()) return false; // no need to spend time on isomorphism ...
		boolean b= (UniversalIsomorphismTester.isIsomorph(mol,q));
		
		if (logger.isDebugEnabled())
			if (b) logger.info(MSG_MOLECULEIS,q.getID(),"\tYES");
			else logger.info(MSG_MOLECULEIS,q.getID(),"\tNO");
		return b;		
		} catch (CDKException x) {
			logger.error(x);
			return false;
		}		

	}
	private static int countGroups(IAtomContainer mol, QueryAtomContainer q) {
		List list = getUniqueBondMap(mol,q,false);
		if (list == null) return 0;
		else if (list.size() == 0) return 0; 
		else {
			markMaps(mol,q,list);
			return 1;
		}
	}
	public static boolean hasManyDifferentFunctionalGroups(IAtomContainer  mol,int threshold) {
		int groups = 0;
		boolean group = false;
		String metals[] = {"Na","K","Ca","Mg","Al"};
		MFAnalyser mf= new MFAnalyser(mol);
		markCHn(mol);
		Vector elements = mf.getElements();
		logger.debug("Checking for more than " + threshold ," groups");
		for (int i=0; i< elements.size(); i++) {
			String element = (String) elements.get(i);
			if (element.equals("H")) continue; 
			if (element.equals("C")) {
				if (isAcetylenic(mol)) groups++;
				continue;
			}
			if (element.equals("N")) {
				group = FunctionalGroups.hasGroup(mol,secondaryAmine(false)) || 
				FunctionalGroups.hasGroup(mol,tertiaryAmine()) ||
				FunctionalGroups.hasGroup(mol,primaryAmine(false));
				if (group) {
					groups++;
					logger.debug("amino group found");
				}
				groups += countGroups(mol,cyano());
				groups += countGroups(mol,nitro1double());
				groups += countGroups(mol,nitro2double());
				groups += countGroups(mol,Nnitroso());
				groups += countGroups(mol,diAzo());
				groups += countGroups(mol,triAzeno());
				if (groups >= threshold) break;
			} else
			
			if (element.equals("O")) {
				int g = countGroups(mol,carboxylicAcid())  + countGroups(mol,FunctionalGroups.ester());
				if (g > 0 ) {
					groups ++; //count once
					logger.debug("acid or ester group found");
				}
				groups += countGroups(mol,saltOfCarboxylicAcid(metals));
				groups += countGroups(mol,acetal());
				groups += countGroups(mol,ether());
				groups += countGroups(mol,aldehyde());
				groups += countGroups(mol,ketone());			
				groups += countGroups(mol,alcohol(false));
				if (groups >= threshold) break;
			} else
			
			if (element.equals("S")) {
				groups += countGroups(mol,sulphide());
				groups += countGroups(mol,mercaptan());
				groups += countGroups(mol,thioester());
				if (groups >= threshold) break;
				groups += countGroups(mol,sulphate(metals));			
				groups += countGroups(mol,sulphamate(metals));
				groups += countGroups(mol,sulphonate(metals));
				if (groups >= threshold) break;
			} else {
				//count each unlisted atom as group ... quite rude ;)
				if (groups >= threshold) break;
				groups++;
			}
			
		}
		logger.info("Different groups found\t",groups);
		return (groups >= threshold);
	}	
	/*
	public static boolean hasManyDifferentFunctionalGroups(org.openscience.cdk.interfaces.AtomContainer  mol,int threshold) {
		int groups = 0;
		boolean group = false;
		String metals[] = {"Na","K","Ca","Mg","Al"};
		MFAnalyser mf= new MFAnalyser(mol);
		markCHn(mol);
		Vector elements = mf.getElements();
		logger.debug("Checking for more than " + threshold ," groups");
		for (int i=0; i< elements.size(); i++) {
			String element = (String) elements.get(i);
			if (element.equals("H")) continue; 
			if (element.equals("C")) {
				if (isAcetylenic(mol)) groups++;
				continue;
			}
			if (element.equals("N")) {
				group = FunctionalGroups.hasGroup(mol,secondaryAmine(false)) || 
				FunctionalGroups.hasGroup(mol,tertiaryAmine()) ||
				FunctionalGroups.hasGroup(mol,primaryAmine(false));
				if (group) {
					groups++;
					logger.debug("amino group found");
				}
				groups += countGroups(mol,cyano());
				groups += countGroups(mol,nitro1double());
				groups += countGroups(mol,nitro2double());
				groups += countGroups(mol,Nnitroso());
				groups += countGroups(mol,diAzo());
				groups += countGroups(mol,triAzeno());
				if (groups >= threshold) break;
			} else
			
			if (element.equals("O")) {
				group = FunctionalGroups.hasGroup(mol,carboxylicAcid()) || 
						
					FunctionalGroups.hasGroup(mol,ester());
				if (group) {
					groups++;
					logger.debug("acid or ester group found");
				}
				groups += countGroups(mol,saltOfCarboxylicAcid(metals));
				groups += countGroups(mol,acetal());
				groups += countGroups(mol,ether());
				groups += countGroups(mol,aldehyde());
				groups += countGroups(mol,ketone());			
				groups += countGroups(mol,alcohol(false));
				if (groups >= threshold) break;
			} else
			
			if (element.equals("S")) {
				groups += countGroups(mol,sulphide());
				groups += countGroups(mol,mercaptan());
				groups += countGroups(mol,thioester());
				if (groups >= threshold) break;
				groups += countGroups(mol,sulphate(metals));			
				groups += countGroups(mol,sulphamate(metals));
				groups += countGroups(mol,sulphonate(metals));
				if (groups >= threshold) break;
			} else {
				//count each unlisted atom as group ... quite rude ;)
				if (groups >= threshold) break;
				groups++;
			}
			
		}
		logger.info("Different groups found\t",groups);
		return (groups >= threshold);
	}
	*/
	public static int associateIonic(IAtomContainer a) throws CDKException {
		
		ISetOfAtomContainers c = ConnectivityChecker.partitionIntoMolecules(a);
		//clear VISITED flag
		for (int i=0; i<a.getAtomCount();i++) a.getAtomAt(i).setFlag(CDKConstants.VISITED,false);
		
		int n = c.getAtomContainerCount();
		if (n == 1) return 0;
		logger.debug("Trying to find ionic bonds (if any)");
		IAtomContainer ac = null;
		ArrayList match = new ArrayList();
		StringBuffer notMatched = new StringBuffer();
		int ionicBonds = 0;
		for (int i=0; i<n; i++) {
			ac = c.getAtomContainer(i);
			logger.debug("AtomContainer \t",Integer.toString(i+1)," atom\t",Integer.toString(ac.getAtomCount()));
			//if there are atoms with formal charge != 0
			for (int k=0; k<ac.getAtomCount(); k++) {
				IAtom atom = ac.getAtomAt(k);
				if (atom.getFormalCharge() != 0) match.add(atom);
			}
			if (match.size() == 0) continue; //nothing to match
			
			for (int j=i+1; j<n; j++) {
				ac = c.getAtomContainer(j);
				for (int k=0; k<ac.getAtomCount(); k++) {
					IAtom atom = ac.getAtomAt(k);
					
					if (atom.getFormalCharge() != 0) {//lookup in match
						Object aCharge = atom.getProperty(ChargeConsumed.CHARGECONSUMED);
						if (aCharge == null) {
							aCharge = new ChargeConsumed(atom);
							atom.setProperty(ChargeConsumed.CHARGECONSUMED,aCharge);
						} else if (((ChargeConsumed)aCharge).getCharge()==0) continue;	
						
						for (int m=0; m<match.size(); m++) {
							Atom mAtom = (Atom) match.get(m);
							if (mAtom.getFormalCharge() == 0) continue;
							
							Object o = mAtom.getProperty(ChargeConsumed.CHARGECONSUMED);
							if (o == null) mAtom.setProperty(ChargeConsumed.CHARGECONSUMED,new ChargeConsumed(mAtom));
							else if (((ChargeConsumed)o).getCharge()==0) continue; //already associated
							
							ChargeConsumed mCharge = (ChargeConsumed)mAtom.getProperty(ChargeConsumed.CHARGECONSUMED);
							
							if (mCharge.compatible((ChargeConsumed) aCharge)) {
								mCharge.consume((ChargeConsumed) aCharge);
								if (a.getBond(atom,mAtom) == null) {
									/*TODO replace MyAssociationBond with Association
									 * once QueryBond matches method accepts ElectronContainer instead of Bond 
									*/
									//Association ionicBond = new Association(atom,mAtom);
									MyAssociationBond ionicBond = new MyAssociationBond(atom,mAtom);
									logger.debug("Association between atom\t",atom.getSymbol(),"\tand\t",mAtom.getSymbol());
								
									a.addElectronContainer(ionicBond);
									ionicBonds ++;
								} else 	logger.debug("Bond already exists : atom\t",atom.getSymbol(),"\tand\t",mAtom.getSymbol());
								break;
							}	
						}
					}	
				}
			}
			//verify if all atoms are matched
			for (int j=0;j<match.size();j++) {
				Object o = ((Atom) match.get(j)).getProperty(ChargeConsumed.CHARGECONSUMED);
				if (o==null) {
					notMatched.append('\t');
					notMatched.append(((Atom)match.get(j)).getSymbol());
				} else if (( (ChargeConsumed) o).getCharge() != 0) {
					notMatched.append('\t');
					notMatched.append(((Atom)match.get(j)).getSymbol());					
				}
			}	
					
		}
		if (notMatched.toString().length()>0)
			throw new CDKException("Can't find an ionic bond for atom(s)\t"+notMatched.toString());
		
		logger.info("Ionic bonds found\t",ionicBonds);
		return ionicBonds;
	}
	/**
	 * Identifies the fragment to be detached and breaks the relevant bonds
	 * In order to function properly, the group has to be defined with at least one atom
	 * marked with  setProperty(DONTMARK,dontMark). This defined the bond to break.
	 * See {@link #DONTMARK}
	 * @param a AtomContainer to be processed
	 * @param q {@link QueryAtomContainer} defines the group to be detached
	 * @return {@link SetOfAtomContainers} set of {@link AtomContainer} resulted
	 * by breaking bonds as above. Hydrogens are added on the broken end of each bond 
	 *TODO care for bond orders!
	 */
	public static ISetOfMolecules detachGroup(IAtomContainer a, QueryAtomContainer q) {
		
		List list = FunctionalGroups.getUniqueBondMap(a, q,needsPreprocessing(q));
		if ((list == null) || (list.size() == 0)) return null;
		markMaps(a,q,list);
		ArrayList bondsToBreak = new ArrayList();
		for (int j = 0; j < list.size(); j++) {
			List first = (List)list.get(j);
			for (int i = 0; i < first.size(); i++) {
				RMap rmap = (RMap)first.get(i);
				IBond b2 = q.getBondAt(rmap.getId2());
				IAtom[] a2 = b2.getAtoms();
				
				//will break bonds that does not belong to the group but to the connected fragments
				//org.openscience.cdk.interfaces.Atom deleteAtom = null;
				IAtom stayAtom = null;
				//if the bond belongs to the group, the stayAtom will remain null
				for (int k=0; k<a2.length;k++) 
					if (a2[k] instanceof AnyAtom) stayAtom = a2[k];
					else {
						Object o = a2[k].getProperty(DONTMARK);
						if ((o != null) && (o.equals(q.getID()))) stayAtom = a2[k];
						//else deleteAtom = a2[k];
					}
				
				if (stayAtom != null)  {
					//gather all bonds to be broken in a list
					IBond b1 = a.getBondAt(rmap.getId1());
					if (bondsToBreak.indexOf(b1) == -1)
						bondsToBreak.add(b1);
					break;
				}
			}
		
		}	
		if (logger.isDebugEnabled()) 
			for (int i=0; i< bondsToBreak.size();i++) {
				Bond b = (Bond)bondsToBreak.get(i);
				logger.debug("Break bond between\t",b.getAtomAt(0).getSymbol(),"\t",b.getAtomAt(1).getSymbol());
			}
		for (int i=0; i< bondsToBreak.size();i++) {
			IBond b = (Bond)bondsToBreak.get(i);
			IAtom[] a1 = b.getAtoms();
			a.removeElectronContainer(b);
			Atom h = null;
			for (int k=0; k<a1.length;k++) {
				if (q.getID().equals(CARBOXYLIC_ACID_SALT))
					if (a1[k].getSymbol().equals("O") && (a1[k].getFormalCharge()==-1)) {
						a1[k].setFormalCharge(0);
						h = new Atom("H");
						a.addAtom(h);
						a.addBond(new org.openscience.cdk.Bond(a1[k],h));
						a1[k].removeProperty(q.getID());
					} else {
						a1[k].setFormalCharge(0);
						//a1[k].removeProperty(DONTMARK);
						//a1[k].setProperty(q.getID(),new Boolean(true));
					}
				else if (q.getID().equals(HYDROCHLORIDE_OF_AMINE)) {
					if (a1[k].getSymbol().equals("N") && (a1[k].getFormalCharge()==1)) {
						a1[k].setFormalCharge(0);
						//remove h
						IAtom[] neighbors = a.getConnectedAtoms(a1[k]);
						IAtom cl = null;
						for (int ih=0;ih<neighbors.length;ih++)
							if (neighbors[ih].getSymbol().equals("H")) {
								logger.debug("a H atom connected to N will be moved to Cl");
								a.removeBond(a1[k],neighbors[ih]);
								cl = a1[(k+1)%2]; //TODO this assumes two atoms in the bond
								if (cl.getSymbol().equals("Cl")) {
									cl.setFormalCharge(0);
									a.addBond(new org.openscience.cdk.Bond(cl,neighbors[ih]));
									neighbors[ih].removeProperty(q.getID());
								} else logger.error("Expected Cl atom but found\t",cl);
								break;
							}
							
					} else;
					/*{
						a1[k].setFormalCharge(0);						
						h = new Atom("H");
						a.addAtom(h);
						a.addBond(new org.openscience.cdk.Bond(a1[k],h));
					}*/
				}else if (q.getID().equals(LACTONE)) {
					if (a1[k].getSymbol().equals("O")) {
						h = new Atom("H");
						a.addAtom(h);
						a.addBond(new org.openscience.cdk.Bond(a1[k],h));
						a1[k].removeProperty(q.getID());
					} else { // atatch OH
						h = new Atom("H");
						a.addAtom(h);
						Atom o = new Atom("O");
						a.addAtom(o);
						a.addBond(new org.openscience.cdk.Bond(o,h));
						a.addBond(new org.openscience.cdk.Bond(a1[k],o));						
	
					}
				} else {	
					h = new Atom("H");
					a.addAtom(h);
					a.addBond(new org.openscience.cdk.Bond(a1[k],h));
				}
			}
		}
		if ( bondsToBreak.size() > 0)
			return ConnectivityChecker.partitionIntoMolecules(a);
		else return null;
	}
	public static void clearMark(IAtomContainer a, QueryAtomContainer q) {
		clearMark(a,q.getID());
	}	
	public static void clearMark(IAtomContainer a, Object id) {
		IBond [] b = a.getBonds();

		for (int i=0; i < b.length;i++) {
			b[i].removeProperty(id);
			b[i].removeProperty(ALLOCATED);			
			b[i].removeProperty(DONTMARK);
			IAtom[] atoms = b[i].getAtoms();
			for (int k=0; k < atoms.length;k++) {
				atoms[k].removeProperty(id);
				atoms[k].removeProperty(DONTMARK);
				atoms[k].removeProperty(ALLOCATED);
			}
		}
		
	}
	/**
	 * 
	 * @param size
	 * @return ring
	 */
	public static QueryAtomContainer ring(int size) {
		QueryAtomContainer q = new QueryAtomContainer();
		q.setID("RING "+Integer.toString(size));
		SymbolQueryAtom[] c= new SymbolQueryAtom[size];
		AnyAtom[] h= new AnyAtom[size];
		for (int i=0;i < size; i++) {
			c[i] = new SymbolQueryAtom(new Atom("C"));
			c[i].setProperty(RING_NUMBERING,new Integer(i));
			q.addAtom(c[i]);
			if (i>0) q.addBond(new AnyOrderQueryBond(c[i],c[i-1],1));
			h[i] = new AnyAtom();
			h[i].setProperty(RING_NUMBERING,new Integer(i));
			q.addAtom(h[i]);
			q.addBond(new AnyOrderQueryBond(c[i],h[i],1));
		}
		q.addBond(new AnyOrderQueryBond(c[0],c[size-1],1));
		return q;
	}
	/**
	 * 
	 * @param mol
	 * @param q
	 */
	public static void markAtomsInRing(IAtomContainer mol, QueryAtomContainer q) {
		try {
			List list = UniversalIsomorphismTester.getSubgraphAtomsMap(mol,q);
			if (list != null)
				for (int i=0; i < list.size();i++) {
					RMap map = (RMap)list.get(i);
					Object p = q.getAtomAt(map.getId2()).getProperty(RING_NUMBERING);
					mol.getAtomAt(map.getId1()).setProperty(RING_NUMBERING,p);
				}
			
		} catch (CDKException x) {
			logger.debug(x);
		}
		
	}

	/**
	 * 
	 * @param mol {@link AtomContainer}
	 * @return the length of the longest carbon chain
	 */
	public static int getLongestCarbonChainLength(IAtomContainer mol) {
		double[][] conMat = ConnectionMatrix.getMatrix(mol);
		int[][] apsp = PathTools.computeFloydAPSP(conMat);
		int maxPathLength = 0;
		int bestStartAtom = -1;
		int bestEndAtom = -1;
		IAtom atom = null;
		IAtomContainer path = new org.openscience.cdk.AtomContainer();
		
		for (int f = 0; f < apsp.length; f++) 	{
			atom = mol.getAtomAt(f);
			if (mol.getBondCount(atom) == 1) 	{
				for (int g = 0; g < apsp.length; g++) 	{
					if (apsp[f][g] > maxPathLength) {
						try {
							path.removeAllElements();
							for (int i=0;i< mol.getAtomCount();i++)
								mol.getAtomAt(i).setFlag(CDKConstants.VISITED,false);							
							if (PathTools.depthFirstTargetSearch(
									(org.openscience.cdk.AtomContainer)mol,
									(org.openscience.cdk.Atom)mol.getAtomAt(f),
									(org.openscience.cdk.Atom)mol.getAtomAt(g),path)) {
								
								int c =0;
								for (int i=0;i< path.getAtomCount();i++)
									if (path.getAtomAt(i).getSymbol().equals("C")) c++;
								
								if (c > maxPathLength) {
									maxPathLength = c;
									bestStartAtom = f;
									bestEndAtom = g;
								}
							}
						} catch (NoSuchAtomException x) {
							x.printStackTrace();
						}
					}
				}
			}
		}
		for (int i=0;i< mol.getAtomCount();i++)
			mol.getAtomAt(i).setFlag(CDKConstants.VISITED,false);
		logger.debug("Longest chaing in molecule is of length " + maxPathLength + " between atoms " + (bestStartAtom+1) +  " and " + (bestEndAtom+1) );
		
		return maxPathLength;
	}

	public static QueryAtomContainers getAllGroups() {
		String[] metals = {"Na","K","Ca"};
		QueryAtomContainers list = new QueryAtomContainers();
		IAtomContainer m = new org.openscience.cdk.AtomContainer();
		m.addAtom(new Atom("C"));
		m.setID("Empty fragment");
		list.add(m);
		list.add(methyl());
		list.add(methoxy());
		list.add(ethyl());
		list.add(primaryAmine(false));
		list.add(secondaryAmine(false));
		list.add(tertiaryAmine());
		list.add(hydrochlorideOfAmine(1));
		list.add(sulphateOfAmine(1));
		list.add(cyano());
		list.add(nitro2double());
		list.add(nitro1double());
		list.add(Nnitroso());
		list.add(diAzo());
		list.add(triAzeno());
		list.add(quaternaryNitrogen1(false));
		list.add(carboxylicAcid());
		list.add(acyclic_acetal());
		list.add(acetal());
		list.add(ether());
		list.add(sulphide());
		list.add(mercaptan());
		list.add(thioester());
		list.add(ester());
		list.add(carbonate());
		list.add(anhydride());
		list.add(lactone(false));
		list.add(ketone());
		list.add(aldehyde());
		list.add(alcohol(false));
		list.add(sulphamate(metals));
		list.add(saltOfCarboxylicAcid(metals));
		list.add(sulphonate(metals));
		list.add(sulphate(metals));
		return list;
	}
	
}
/**
 * 
 * Used when recognising ionic bonds
 * @author Nina Jeliazkova
 * <b>Modified</b> 2005-10-22
 */
class ChargeConsumed {
	protected IAtom atom;
	public static String CHARGECONSUMED = "ChargeConsumed";
	protected int charge=0;
	public ChargeConsumed(IAtom atom) {
		this.atom = atom;
		this.charge = atom.getFormalCharge();
	}
	public void consume(int amount) {
		charge = charge + amount;
	}
	public void consume(ChargeConsumed amount) {
		if (charge == - amount.getCharge()) {
			int newCharge = charge + amount.getCharge();
			amount.consume(charge);
			charge = newCharge;
		} else {
			int minCharge = charge;
			if (Math.abs(amount.getCharge()) < Math.abs(charge)) 
				minCharge = -amount.getCharge();
			
			amount.consume(minCharge);
			consume(-minCharge);
		}
		
		//System.out.println(toString());
		//System.out.println(amount.toString());
	}	
	public int getCharge() {
		return charge;
	}
	public boolean compatible(ChargeConsumed c) {
		return (this.charge * c.getCharge()) < 0;
	}
	public String toString() {
		return atom.getSymbol() + "\tFormalCharge=" + atom.getFormalCharge() + "\tCharge Left=" + charge;
	}
	
}

			